from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class FleetDevelopmentVessels(KplerClient):

    """
    The ``FleetDevelopmentVessels`` endpoint provides In-Service fleet, new vessel deliveries and old vessels sold for scrap as well as vessels with Scrubbers installed and planned.
    """

    RESOURCE_NAME = "fleet-development/vessels"
    RESOURCE_GET_COLUMN_NAME = "fleet-development"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the fleet_development endpoints

        Examples:
            >>> from kpler.sdk.resources.fleet_development_vessels import FleetDevelopmentVessels
            ... fleet_development_vessels_client=FleetDevelopmentVessels(config)
            ... fleet_development_vessels_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "day","Day","None","False","date yyyy-MM-dd"
                "vessel_name","Name","Name of the vessel","False","string"
                "vessel_imo","IMO","Vessel IMO","False","string"
                "vessel_mmsi","MMSI","The Maritime Mobile Service Identity of the vessel (9 digits)","False","string"
                "vessel_status","Status","Status of the vessel","False","string"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_GET_COLUMN_NAME)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        metric: Enum = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        compliance_methods: Optional[List[Enum]] = None,
        columns: Optional[List[str]] = None,
        size: Optional[int] = None,
        vessel_types: Optional[List[Enum]] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        with_orderbook: bool = False,
        vessels: Optional[List[str]] = None,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            metric: Enum ``FleetDevelopmentVesselsMetric``
            vessel_types_cpp: Optional[List[Enum]] ``VesselTypesCPP``
            vessel_types_oil: Optional[List[Enum]] ``VesselTypesOil``
            compliance_methods: Optional[List[Enum]] ``FleetDevelopmentVesselsComplianceMethods``
            columns: Optional[List[str]] Retrieve all available columns when set to "all"
            size:  Optional[int] Maximum number of fleet development vessels returned
            vessel_types: Optional[Enum] ``VesselTypesDry`` ``VesselTypesLNG`` ``VesselTypesLPG``
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            with_orderbook: bool = False Access vessels in fleet orderbook, scheduled for future delivery
            vessels: Optional[List[str]] Names or IMOs of the vessels

        Examples:

            >>> from datetime import date
            ... from kpler.sdk.resources.fleet_development_vessels import FleetDevelopmentVessels
            ... from kpler.sdk import FleetDevelopmentVesselsMetric, FleetDevelopmentVesselsComplianceMethods
            ... fleet_development_vessels_client=FleetDevelopmentVessels(config)
            ... fleet_development_vessels_client.get(
            ...     start_date=date(2020,11,1),
            ...     end_date=date(2020,12,1),
            ...     metric=FleetDevelopmentVesselsMetric.Available,
            ...     compliance_methods=[FleetDevelopmentVesselsComplianceMethods.Scrubber],
            ...     size=5
            ... )

            .. csv-table::
                :header:  "Day","Name","IMO","MMSI","Status","Build year","Carrier type","Flag name","Engine type","Cargo system","..."

                "2020-11-01","Marlin Modest","9833577","563087800","Active","2019","NaN","Singapore","NaN","NaN","..."
                "2020-11-01","Silver Emily","9682356","538005680","Active","2014","NaN","Marshall Islands","NaN","NaN","..."
                "2020-11-01","Trf Mobile","9732802","538006545","Active","2016","NaN","Marshall Islands","NaN","NaN","..."
                "2020-11-01","Plover Pacific","9399911","563037200","Active","2009","NaN","Marshall Islands","NaN","NaN","..."
                "2020-11-01","Torm Atlantic","9433509","566428000","Active","2010","NaN","Singapore","NaN","NaN","..."
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "metric": process_enum_parameter(metric),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "complianceMethods": process_enum_parameters(compliance_methods, False),
            "columns": process_list_parameter(columns),
            "size": size,
            "vesselTypes": process_enum_parameters(vessel_types),
            "gte": gte,
            "lte": lte,
            "withOrderbook": process_bool_parameter(with_orderbook),
            "vessels": process_list_parameter(vessels),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import process_list_parameter


class Zones(KplerClient):
    """
    The ``Zones`` endpoint allows to perform full-text search on zones,
    in order to find names used in Kpler referential.
    """

    RESOURCE_NAME = "zones"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint zones.

        Examples:
            >>> from kpler.sdk.resources.zones import Zones
            ... zones_client = Zones(config)
            ... zones_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "ancestor_id","Ancestor Id","Identifier in the database of Kpler","False","long"
                "ancestor_name","Ancestor Name","Name of the Ancestor Zone","False","string"
                "ancestor_type","Ancestor Type","Ancestor Zone Type","False","string"
                "descendant_id","Descendant Id","Identifier in the database of Kpler","False","long"
                "descendant_name","Descendant Name","Name of the Descendant Zone","False","string"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        columns: Optional[List[str]] = None,
        ancestor_id: Optional[List[int]] = None,
        ancestor_name: Optional[List[str]] = None,
        ancestor_type: Optional[List[str]] = None,
        descendant_id: Optional[List[int]] = None,
        descendant_name: Optional[List[str]] = None,
        descendant_type: Optional[List[str]] = None,
    ) -> DataFrame:
        """

        Args:
            ancestor_id:  Optional[int] IDs ancestor zones. Can be separated by a coma.
            ancestor_name:  Optional[str] Names ancestor zones. Can be separated by a coma.
            ancestor_type:  Optional[str] Type of ancestor zones. Can be separated by a coma. e.g. continent, subcontinent, region, subregion, country, port, custom, bay, cape, gulf, ocean, sea, strait, ...
            descendant_id:  Optional[str] IDs descendant zones. Can be separated by a coma.
            descendant_name:  Optional[int] Names descendant zones. Can be separated by a coma.
            descendant_type:  Optional[int] Type of descendant zones. Can be separated by a coma. e.g. continent, subcontinent, region, subregion, country, port, custom, bay, cape, gulf, ocean, sea, strait, ...
            columns: Optional[List[str]] Retrieve all available columns when set to "all"

        Examples:
            >>> from kpler.sdk.resources.zones import Zones
            ... zones_client = Zones(config)
            ... zones_client.get(ancestor_id=[515])

            .. csv-table::
                :header: "ancestor_id","ancestor_name", "ancestor_type","descendant_id","descendant_name","descendant_type"

                "515;Latin America","custom","209","Chile","country"
                "515","Latin America","custom","283","East Coast Mexico","subregion"
                "515","Latin America","custom","177","Caribbean Islands","subcontinent"
                "515","Latin America","custom","187","Cayman Islands","country"
                "515","Latin America","custom","239","Curacao","country"
                "...","...","...","..."
        """
        query_parameters = {
            "ancestorId": process_list_parameter(ancestor_id),
            "ancestorName": process_list_parameter(ancestor_name),
            "ancestorType": process_list_parameter(ancestor_type),
            "descendantId": process_list_parameter(descendant_id),
            "descendantName": process_list_parameter(descendant_name),
            "descendantType": process_list_parameter(descendant_type),
            "columns": process_list_parameter(columns),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)

    def search(self, q: str) -> DataFrame:
        """

        Args:
            q: str Argument to search by in zones names

        Examples:
            >>> from kpler.sdk.resources.zones import Zones
            ... zones_client=Zones(config)
            ... zones_client.search("oecd europe")

            .. csv-table::
                :header: "zones"

                "OECD Europe"
                "NON-OECD Europe"

        """
        query_parameters = {"q": q, "resources": self.RESOURCE_NAME}
        return self._search(query_parameters)



from datetime import datetime
from typing import Any, Dict, List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_list_parameter,
    process_updates_date_parameter
)


class TradesUpdates(KplerClient):
    """
    The ``TradesUpdates`` query returns a updates of our trades data at a specified time.
    """

    RESOURCE_NAME = "trades/updates"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = False, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        products: Optional[List[str]] = None,
        start_date: Optional[datetime] = None,
        show_all_history: Optional[str] = "false",
        columns: Optional[List[str]] = None,
        return_columns_ids: Optional[bool] = False,
    ) -> DataFrame:
        """

        Args:
            products: Optional[List[str]] Names of products
            start_date: Optional[datetime] The starting point in time of the returned "diffs", in UTC 0 time. All diffs that have been computed strictly after that date will be returned. Note: a time window of 15 days of diffs are kept, you can't look before in the past
            show_all_history: Optional [String]: Use ['true', 'false']. Default to false if not specified.
            columns: Optional[List[str]] Columns in the jsonUpdates field, Retrieve all available columns when set to "all"
            return_columns_ids: Optional[bool] set it to True to use columns ids instead of names in the jsonUpdates column. Default to False if not specified.
        Examples:
            >>> from datetime import date, timedelta, datetime
            ... from kpler.sdk.resources.trades_updates import TradesUpdates
            ... trades_updates_client = TradesUpdates(config)
            ... updates = trades_updates_client.get(
            ... products = ["crude"],
            ... start_date = datetime.strptime('2023-07-21T160000', '%Y-%m-%dT%H%M%S'),
            ... columns = [
            ...         "vessel_name",
            ...         "origin_location_name",
            ...         "destination_location_name",
            ...         "start",
            ...         "end"
            ... ]
            ... )
            >>> print(updates.headers['lastAvailableDate'])
            "2022-10-19T034600"

            .. csv-table::
                :header: "productId", "tradeId", "date", "operation", "jsonUpdates"

                "1096", "14686898", "2022-09-21 14:01:00", "UPDATE", "{'Date (destination)':'2022-09-21 02:30','Dest..."
                "1780", "14870244", "2022-09-21 14:01:00", "UPDATE", "{'Date (destination)':'2022-09-28 12:14'}"
                "1248", "15141376", "2022-09-21 14:01:00", "UPDATE", "{'Date (destination)':'2022-09-21 10:00','Dest..."
                "1280", "15269441", "2022-09-21 14:01:00", "INSERT", "{'Vessel':'San Matias I','Date (origin)':'2022..."
                "1932", "15269510", "2022-09-21 14:01:00", "INSERT", "{'Vessel':'Aretea','Date (origin)':'2022-10-17..."
                "...", "...", "...", "...", "..."

        """

        query_parameters: Dict[str, Optional[Any]] = {
            "products": process_list_parameter(products),
            "startDate": process_updates_date_parameter(start_date),
            "showAllHistory": process_bool_parameter(show_all_history),
            "columns": process_list_parameter(columns),
        }
        header_parameters: List[str] = ["lastAvailableDate"]

        self.column_ids = return_columns_ids

        return self._get_dataframe(self.RESOURCE_NAME, query_parameters, header_parameters)



from datetime import date
from typing import List, Optional

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import process_date_parameter, process_list_parameter


class InventoriesTankLevels(KplerClient):

    """
    The `InventoriesTankLevels` endpoint provides with the level of each and every tank in the indicated installation or zone, on a configurable period of maximum 31 days.
    """

    RESOURCE_NAME = "inventories/tank-levels"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        installations: Optional[List[str]] = None,
        zones: Optional[List[str]] = None,
    ):
        """
        Args:
            start_date:  Optional[date] Start of the period (YYYY-MM-DD)
            end_date:  Optional[date] End of the period (YYYY-MM-DD)
            installations: Optional[List[str]] Names of installations
            zones: Optional[List[str]] Names of countries/geographical zones

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.inventories_tank_levels import InventoriesTankLevels
            ... inventories_tank_levels_client=InventoriesTankLevels(config)
            ... inventories_tank_levels_client.get(
            ...     start_date=date(2020,11,1),
            ...     end_date=date(2020,11,1),
            ...     zones=["Japan"]
            ... )

            .. csv-table::
                :header: "Country","Installation","Tank","Date","Level","Capacity","Capacity","Utilization","Tank","Type"

                "Japan","Kamigoto Storage","JP_KAM.001","2020-11-01","2.337356e+07","2.767516e+07","0.844568","spr"
                "Japan","Kikuma Underground","JP_KIK.001","2020-11-01","7.565012e+06","8.900012e+06","0.850000","spr"
                "Japan","Kuji","JP_KUJ.001","2020-11-01","8.925014e+06","1.050001e+07","0.850000","spr"
                "Japan","Kushikino National Petroleum Stockpiling Base","JP_KUS.001","2020-11-01","8.925014e+06","1.050001e+07","0.850000","spr"
                "Japan","White Island","JP_WHI.001","2020-11-01","3.330794e+07","3.522294e+07","0.945632","spr"
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration


class Players(KplerClient):
    """
    The ``Players`` endpoint allows to perform full-text search on players, in order to find names used in Kpler referential.
    """

    RESOURCE_NAME = "players"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def search(self, q: str) -> DataFrame:
        """

        Args:
            q: str Argument to search by in players names

        Examples:
            >>> from kpler.sdk.resources.players import Players
            ... players_client=Players(config)
            ... players_client.search("somo")

            .. csv-table::
                :header: "players"

                "SOMO"
                "Somoil"
        """
        query_parameters = {"q": q, "resources": self.RESOURCE_NAME}
        return self._search(query_parameters)



from datetime import date
from enum import Enum
from typing import Optional

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import process_date_parameter, process_enum_parameter


class InventoriesCushingDrone(KplerClient):

    """
    The `InventoriesCushingDrone` endpoint returns crude inventories of Cushing for a specific  periodicity over a chosen period of time.
    """

    RESOURCE_NAME = "inventories/cushing-drone"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        period: Optional[Enum] = None,
    ):
        """
        Args:
            start_date:  Optional[date] Start of the period (YYYY-MM-DD)
            end_date:  Optional[date] End of the period (YYYY-MM-DD)
            period: Optional[Enum] ``InventoriesDronePeriod``

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.inventories_cushing_drone import InventoriesCushingDrone
            ... from kpler.sdk.enums import InventoriesDronePeriod
            ... inventories_tank_levels_client=InventoriesCushingDrone(config)
            ... inventories_tank_levels_client.get(
            ...     start_date=date(2021,6,14),
            ...     end_date=date(2021,8,14),
            ...     period=InventoriesDronePeriod.EiaWeeks
            ... )

            .. csv-table::
                :header: "Date","Level (kb)","Capacity (kb)","Capacity Utilization",

                "2021-06-18","45695","96947","0.471"
                "2021-06-25","44200","96947","0.456"
                "2021-07-02","43685","96947","0.451"
                "2021-07-09","42424","96947","0.438"
                "2021-07-16","40684","96947","0.420"
                "2021-07-23","39366","97353","0.404"
                "2021-07-30","39993","97353","0.401"
                "2021-08-06","39652","97353","0.407"
                "2021-08-13","37919","97569","0.389"
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "period": process_enum_parameter(period),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class FreightMetricsSeries(KplerClient):

    """
    The ``FreightMetricsSeries`` endpoint returns demand metrics like tonmiles & tondays as well as other metrics like average speed for laden fleet and average distance under this metric
    """

    RESOURCE_NAME = "freight-metrics/series-alt"
    COLUMNS_RESOURCE_NAME = "freight-metrics"
    COLUMNS_PATH = "columns-alt"

    AVAILABLE_PLATFORMS = [Platform.Liquids, Platform.LPG, Platform.LNG, Platform.Dry]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint freight-metrics.


        Examples:
            >>> from kpler.sdk.resources.freight_metrics_series import FreightMetricsSeries
            ... freight_metrics_series_client = FreightMetricsSeries(config)
            ... freight_metrics_series_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date (timestamp)","","False","string"
                "vessel_imo","IMO","IMO of the vessel","False","long"
                "vessel","Name","Name of the vessel","False","string"
                "vessel_dwt_tons","Dead Weight Tonnage","Dead Weight Tonnage","False","long"
                "vessel_type","Vessel Type","Type of the vessel","False","string"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.COLUMNS_RESOURCE_NAME, self.COLUMNS_PATH)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        from_locations: Optional[List[str]] = None,
        to_locations: Optional[List[str]] = None,
        from_installations: Optional[List[str]] = None,
        to_installations: Optional[List[str]] = None,
        metric: Optional[Enum] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        products: Optional[List[str]] = None,
        period: Optional[Enum] = None,
        split: Optional[Enum] = None,
        unit: Optional[Enum] = None,
        with_intra_country: Optional[bool] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        vessel_types: Optional[List[Enum]] = None,
        with_freight_view: bool = False,
        avg_speed_min: Optional[int] = None,
        avg_speed_max: Optional[int] = None,
        status: Optional[Enum] = None,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            from_locations: Optional[str] Names of locations
            to_locations: Optional[str] Names of locations
            from_installations: Optional[str] Names of installations
            to_installations: Optional[str] Names of installations
            metric: Optional[Enum] ``FreightMetricsSeriesMetric``
            vessel_types_cpp: Optional[Enum] ``VesselTypesCPP``
            vessel_types_oil: Optional[Enum] ``VesselTypesOil``
            products: Optional[str] Names of products
            period: Optional[Enum] ``FreightMetricsSeriesPeriod``
            split: Optional[Enum] ``FreightMetricsSeriesSplit``
            unit: Optional[Enum] ``FreightMetricsSeriesUnit``
            with_intra_country: Optional[bool] By default: false. Takes into account the freight metric series within the selected region. Use ["true", "false"]
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            vessel_types: Optional[Enum] ``VesselTypesDry`` ``VesselTypesLNG`` ``VesselTypesLPG``
            with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.
            avg_speed_min: Optional[int] Get vessel legs with average speed greater or equal to this value
            avg_speed_max: Optional[int] Get vessel legs with average speed less than or equal to this value
            status: Optional[Enum] ``FreightMetricsSeriesStatus``

        Examples:

            >>> from datetime import date
            ... from kpler.sdk.resources.freight_metrics_series import FreightMetricsSeries
            ... from kpler.sdk import FreightMetricsSeriesMetric, FreightMetricsSeriesSplit, FreightMetricsSeriesUnit, FreightMetricsSeriesPeriod
            ... freight_metrics_series_client = FreightMetricsSeries(config)
            ... freight_metrics_series_client.get(
            ...     metric=FreightMetricsSeriesMetric.TonMiles,
            ...     split=FreightMetricsSeriesSplit.VesselTypeCpp,
            ...     products=["gasoline", "DPP"],
            ...     unit=FreightMetricsSeriesUnit.KTNMI,
            ...     period=FreightMetricsSeriesPeriod.Monthly,
            ...     start_date=date(2020,1,1)
            ... )

            .. csv-table::
                :header:  "Date","MR","GP","LR2","LR1","LR3","VLCC"

                "2020-01-01","47167787.97","4805725.69","46909311.58","20111132.52","25888954.41","1212255.41"
                "2020-02-01","44779639.94","4799508.63","48502313.55","19338796.56","18212888.97","1907541.54"
                "2020-03-01","53292249.51","5701420.14","46698177.92","22726753.15","14412986.37","3788115.98"
                "2020-04-01","41957007.33","5138045.73","48438084.70","20037436.18","11140717.43","2933625.42"
                "2020-05-01","43838067.51","5344708.32","50284110.11","13431154.99","13004547.35","3421934.68"
                "2020-06-01","42789911.76","4894000.27","41481731.47","13896678.66","13061101.23","3511137.65"
                "2020-07-01","45617055.75","5043752.13","46691881.74","21080645.36","10519866.61","2661771.76"
                "2020-08-01","44716307.74","4320194.82","48217646.71","16982323.71","9916214.49","5142621.42"
                "2020-09-01","43714237.43","4754961.40","38954591.64","17309605.01","15690640.13","784688.29"
                "2020-10-01","42514077.39","4729507.01","42225374.83","17845363.26","15703871.72","3513174.00"
                "2020-11-01","15881015.99","1990767.03","17269526.99","5727516.67","8022010.01","1737537.15"
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "fromLocations": process_list_parameter(from_locations),
            "toLocations": process_list_parameter(to_locations),
            "fromInstallations": process_list_parameter(from_installations),
            "toInstallations": process_list_parameter(to_installations),
            "metric": process_enum_parameter(metric),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil),
            "products": process_list_parameter(products),
            "period": process_enum_parameter(period),
            "split": process_enum_parameter(split),
            "unit": process_enum_parameter(unit),
            "withIntraCountry": process_bool_parameter(with_intra_country),
            "gte": gte,
            "lte": lte,
            "vesselTypes": process_enum_parameters(vessel_types),
            "withFreightView": process_bool_parameter(with_freight_view),
            "avgSpeedMin": avg_speed_min,
            "avgSpeedMax": avg_speed_max,
            "status": process_enum_parameter(status),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class FleetMetrics(KplerClient):

    """
    The ``FleetMetrics`` endpoint provides you with aggregated fleet data per day or week, either the floating storage or all
    the loaded vessels.
    """

    RESOURCE_NAME = "fleet-metrics"

    AVAILABLE_PLATFORMS = [Platform.Liquids, Platform.LPG, Platform.LNG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        zones: Optional[List[str]] = None,
        metric: Optional[Enum] = None,
        period: Optional[Enum] = None,
        floating_storage_duration_min: Optional[str] = None,
        floating_storage_duration_max: Optional[str] = None,
        products: Optional[List[str]] = None,
        unit: Optional[Enum] = None,
        split: Optional[Enum] = None,
        with_freight_view: bool = False,
        vessel_types: Optional[List[Enum]] = None,
        vessel_types_alt: Optional[List[Enum]] = None,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            zones: Optional[List[str]] Names of countries/geographical zones
            metric: Optional[Enum] ``FleetMetricsAlgo``
            period: Optional[Enum] ``FleetMetricsPeriod``
            floating_storage_duration_min: Optional[str] Minimum floating days [7-10-12-15-20-30-90] or [1-3-5-7-10-12-15-20-30-90] on LNG platform
            floating_storage_duration_max: Optional[str] Maximum floating days [7-10-12-15-20-30-90-Inf] or [1-3-5-7-10-12-15-20-30-90-Inf] on LNG platform
            products: Optional[List[str]] Names of products. Not available on LNG platform
            unit: Optional[Enum] ``FleetMetricsMeasurementUnit`` by default return in bbl unit or cm on LNG platform
            split: Optional[Enum] ``FleetMetricsSplit``
            with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.
            vessel_types: Optional[List[Enum]] = Vessel types, pick types based on platform.
            vessel_types_alt: Optional[List[Enum]] = For oil vessel types, only available for Liquids

        Examples:

            >>> from kpler.sdk.resources.fleet_metrics import FleetMetrics
            ... from kpler.sdk import FleetMetricsAlgo, FleetMetricsSplit, FleetMetricsPeriod
            ... fleet_metrics_client = FleetMetrics(config)
            ... fleet_metrics_client.get(
            ...     metric=FleetMetricsAlgo.LoadedVessels,
            ...     zones=["China"],
            ...     period=FleetMetricsPeriod.Daily,
            ...     split=FleetMetricsSplit.CurrentSubregions
            ... )

             .. csv-table::
                :header:  "Date","Hubei","Jiangsu Province","Anhui","Shanghai Province","Liaoning","Zhejiang","Hainan","Hunan","Fujian","..."

                "2019-10-27","0.0","5162437.50","0.00","2701888.77","11170477.14","13718894.53","713446.92","0.0","3467654.33","..."
                "2019-10-28","0.0","4570962.59","0.00","3004283.99","10789081.90","13882417.02","1083772.10","0.0","2644236.36","..."
                "2019-10-29","0.0","6341355.57","149716.36","4651774.07","11371285.63","14548268.95","1672749.96","0.0","4543149.04","..."
                "2019-10-30","0.0","4157074.18","0.00","2306649.65","14003942.45","13719347.39","1443052.37","0.0","2017324.65","..."
                "2019-10-31","0.0","4251597.45","0.00","4825448.32","10310188.30","13959762.82","3180612.54","0.0","2523402.81","..."
                "..","..","..","..","..","..","..","..","..","..",".."
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "zones": process_list_parameter(zones),
            "metric": process_enum_parameter(metric),
            "period": process_enum_parameter(period),
            "floatingStorageDurationMin": floating_storage_duration_min,
            "floatingStorageDurationMax": floating_storage_duration_max,
            "products": process_list_parameter(products),
            "unit": process_enum_parameter(unit),
            "split": process_enum_parameter(split),
            "withFreightView": process_bool_parameter(with_freight_view),
            "vesselTypes": process_enum_parameters(vessel_types, to_lower_case=False),
            "vesselTypesAlt": process_enum_parameters(vessel_types_alt, to_lower_case=False),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class BallastCapacityPortCalls(KplerClient):

    """
    The ``BallastCapacityPortCalls`` endpoint provides all the list of ballast vessels on the input period with their next load ETA and Destination.
    """

    RESOURCE_NAME = "ballast-capacity/port-calls"
    COLUMNS_RESOURCE_NAME = "ballast-capacity"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint ballast-capacity.


        Examples:
            >>> from kpler.sdk.resources.ballast_capacity_port_calls import BallastCapacityPortCalls
            ... ballast_capacity_port_calls_client = BallastCapacityPortCalls(config)
            ... ballast_capacity_port_calls_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date (timestamp)","Date (timestamp)","False","date yyyy-MM-dd"
                "vessel_imo","IMO","Vessel IMO","False","long"
                "vessel","Name","Vessel name","False","string"
                "vessel_dwt_tons","Capacity","Dead Weight Tonnage","False","long"
                "cargo_tons","Cargo (t)","Cargo (t)","False","double"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.COLUMNS_RESOURCE_NAME)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        zones: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        sources: Optional[List[Enum]] = None,
        vessel_state: Optional[List[Enum]] = None,
        size: Optional[int] = None,
        vessel_types: Optional[List[Enum]] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        with_freight_view: bool = False,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            zones: Optional[List[str]] Names of countries/geographical zones
            products: Optional[List[str]] Names of products
            vessel_types_cpp: Optional[List[Enum]] ``VesselTypesCPP``
            vessel_types_oil: Optional[List[Enum]] ``VesselTypesOil``
            sources: Optional[List[Enum]] ``BallastCapacityPortCallsSources``
            vessel_state: Optional[List[Enum]] ``BallastCapacityPortCallsVesselStates``
            size: Optional[int] Maximum number of ballast capacity port calls returned
            vessel_types: Optional[List[Enum]] ``VesselTypesDry`` ``VesselTypesLNG`` ``VesselTypesLPG``
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.

        Examples:
            >>> from kpler.sdk.resources.ballast_capacity_port_calls import BallastCapacityPortCalls
            ... ballast_capacity_port_calls_client = BallastCapacityPortCalls(config)
            ... ballast_capacity_port_calls_client.get(
            ...         zones=["Japan"],
            ...         products=["crude/co"]
            ... )

            .. csv-table::
                :header:  "Date (timestamp)","IMO","Name","Dead Weight Tonnage","Cargo (t)","Next Installation","Next Port","Next Country","Next zone","Next ETA","..."

                "2020-11-24","9868132","Bei Hai Qi Lin","69900.0","75891.0","Kiire","Kiire","Japan","Asia","2020-12-18","..."
                "2020-11-24","9868132","Bei Hai Qi Lin","69900.0","75891.0","Kiire","Kiire","Japan","Asia","2020-12-08","..."
                "2020-11-24","9868132","Bei Hai Qi Lin","69900.0","75891.0","Kiire","Kiire","Japan","Asia","2020-11-28","..."
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "zones": process_list_parameter(zones),
            "products": process_list_parameter(products),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "sources": process_enum_parameters(sources, False),
            "vesselState": process_enum_parameters(vessel_state, False),
            "size": size,
            "vesselTypes": process_enum_parameters(vessel_types),
            "gte": gte,
            "lte": lte,
            "withFreightView": process_bool_parameter(with_freight_view),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from .enums import (
    BallastCapacityPortCallsSources,
    BallastCapacityPortCallsVesselStates,
    BallastCapacitySeriesMetric,
    BallastCapacitySeriesPeriod,
    BallastCapacitySeriesSplit,
    BallastCapacitySeriesUnit,
    CongestionSeriesMetric,
    CongestionSeriesOperation,
    CongestionSeriesPeriod,
    CongestionSeriesSplit,
    CongestionSeriesUnit,
    CongestionVesselsOperation,
    CongestionVesselsPeriod,
    ContractsTypes,
    DiversionsVesselState,
    FleetDevelopmentSeriesAggregationMetric,
    FleetDevelopmentSeriesMetric,
    FleetDevelopmentSeriesPeriod,
    FleetDevelopmentSeriesSplit,
    FleetDevelopmentSeriesUnit,
    FleetDevelopmentVesselsComplianceMethods,
    FleetDevelopmentVesselsMetric,
    FleetMetricsAlgo,
    FleetMetricsMeasurementUnit,
    FleetMetricsPeriod,
    FleetMetricsSplit,
    FleetMetricsVesselsAlgo,
    FleetMetricsVesselsMeasurementUnit,
    FleetUtilizationSeriesMetric,
    FleetUtilizationSeriesPeriod,
    FleetUtilizationSeriesSplit,
    FleetUtilizationSeriesUnit,
    FleetUtilizationSeriesVesselsState,
    FleetUtilizationVesselsPeriod,
    FleetUtilizationVesselsState,
    FleetUtilizationVesselsUnit,
    FlowsDirection,
    FlowsMeasurementUnit,
    FlowsPeriod,
    FlowsSplit,
    FreightMetricsSeriesMetric,
    FreightMetricsSeriesPeriod,
    FreightMetricsSeriesSplit,
    FreightMetricsSeriesUnit,
    InventoriesPeriod,
    InventoriesSplit,
    OutagesTypes,
    Platform,
    SupplyDemandMetric,
    SupplyDemandSplit,
    TradesStatus,
    VesselTypesCPP,
    VesselTypesDry,
    VesselTypesLNG,
    VesselTypesLPG,
    VesselTypesOil,
    FixturesStatuses,
    SupplyDemandUsBalancesCrude,
    SupplyDemandUsBalancesGasoline,
    SupplyDemandUsBalancesGranularity,
    SupplyDemandUsBalancesProduct,
    RunsQualities,
    RunsSplit,
    RunsUnit,
    RunsGranularity,
    ImportsSplit,
    ImportsGranularity,
    ImportsUnit,
    ImportsMetric,
    UtilizationSplit,
    UtilizationGranularity,
    RefinedProductsGranularity,
    RefinedProductsSplit,
    RefinedProducts,
    RefinedProductsUnit,
    SecondaryFeedUnit,
    SecondaryFeedSplit,
    SecondaryFeedGranularity,
    MarginsGranularity,
    MarginsSplit
)


# expose here just classes that customers might use
__all__ = [
    "BallastCapacityPortCallsSources",
    "BallastCapacityPortCallsVesselStates",
    "BallastCapacitySeriesSplit",
    "BallastCapacitySeriesPeriod",
    "BallastCapacitySeriesMetric",
    "BallastCapacitySeriesUnit",
    "CongestionSeriesSplit",
    "CongestionSeriesPeriod",
    "CongestionVesselsPeriod",
    "CongestionSeriesMetric",
    "CongestionSeriesOperation",
    "CongestionSeriesUnit",
    "CongestionVesselsOperation",
    "Platform",
    "TradesStatus",
    "FleetDevelopmentVesselsComplianceMethods",
    "FleetDevelopmentVesselsMetric",
    "FlowsDirection",
    "FlowsPeriod",
    "FlowsSplit",
    "FlowsMeasurementUnit",
    "FleetDevelopmentSeriesAggregationMetric",
    "FleetDevelopmentSeriesMetric",
    "FleetDevelopmentSeriesPeriod",
    "FleetDevelopmentSeriesSplit",
    "FleetDevelopmentSeriesUnit",
    "FleetMetricsPeriod",
    "FleetMetricsMeasurementUnit",
    "FleetMetricsAlgo",
    "FleetMetricsSplit",
    "FleetMetricsVesselsAlgo",
    "FleetMetricsVesselsMeasurementUnit",
    "FleetUtilizationSeriesMetric",
    "FleetUtilizationSeriesPeriod",
    "FleetUtilizationVesselsPeriod",
    "FleetUtilizationSeriesSplit",
    "FleetUtilizationSeriesUnit",
    "FleetUtilizationSeriesVesselsState",
    "FleetUtilizationVesselsState",
    "FleetUtilizationVesselsUnit",
    "OutagesTypes",
    "ContractsTypes",
    "FreightMetricsSeriesPeriod",
    "FreightMetricsSeriesSplit",
    "FreightMetricsSeriesUnit",
    "FreightMetricsSeriesMetric",
    "VesselTypesCPP",
    "VesselTypesOil",
    "VesselTypesDry",
    "VesselTypesLNG",
    "VesselTypesLPG",
    "InventoriesPeriod",
    "InventoriesSplit",
    "DiversionsVesselState",
    "SupplyDemandMetric",
    "SupplyDemandSplit",
    "FixturesStatuses",
    "SupplyDemandUsBalancesCrude",
    "SupplyDemandUsBalancesGasoline",
    "SupplyDemandUsBalancesGranularity",
    "SupplyDemandUsBalancesProduct",
    "RunsQualities",
    "RunsSplit",
    "RunsUnit",
    "RunsGranularity",
    "ImportsSplit",
    "ImportsGranularity",
    "ImportsUnit",
    "ImportsMetric",
    "UtilizationSplit",
    "UtilizationGranularity",
    "RefinedProductsGranularity",
    "RefinedProductsSplit",
    "RefinedProducts",
    "RefinedProductsUnit",
    "SecondaryFeedUnit",
    "SecondaryFeedSplit",
    "SecondaryFeedGranularity",
    "MarginsGranularity",
    "MarginsSplit"
]



from datetime import date
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import process_bool_parameter, process_date_parameter, process_list_parameter


class ShipToShips(KplerClient):
    """
    The ``ShipToShips`` endpoint returns the ship-to-ships loadings/discharges in a given zone,with details on the
    vessels taking part in the operation and crude grades being exchanged for a given time period.
    """

    RESOURCE_NAME = "ship-to-ships"

    AVAILABLE_PLATFORMS = [Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint ship-to-ships.

        Examples:
            >>> from kpler.sdk.resources.ship_to_ships import ShipToShips
            ... ship_to_ships_client = ShipToShips(config)
            ... ship_to_ships_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "load_vessel_name","Vessel (load)","Name of the vessel load","False","string"
                "load_vessel_imo","IMO (load)","IMO of the vessel load","False","string"
                "discharge_vessel_name","Vessel (discharge)","Name of the vessel discharge","False","string"
                "discharge_vessel_imo","IMO (discharge)","IMO of the vessel discharge","False","string"
                "flow_quantity_cms","Volume (cm)","None","False","long"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        size: Optional[int] = None,
        vessels: Optional[List[str]] = None,
        zones: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        columns: Optional[List[str]] = None,
        with_freight_view: bool = False,
    ):
        """

        Args:
             size: Optional[int] Maximum number of sts returned
             vessels: Optional[List[str]] Names/IMOs of vessels
             zones: Optional[List[str]] Names of zones ["port", "lightering", "region", "country", "continent"]
             products: Optional[List[str]] Names of products
             start_date: Optional[date] Start of the period (YYYY-MM-DD)
             end_date: Optional[date] End of the period (YYYY-MM-DD)
             columns: Optional[List[str]] Retrieve all available columns when set to "all"
             with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.

        Examples:
            >>> from kpler.sdk.resources.ship_to_ships import ShipToShips
            ... ship_to_ships_client = ShipToShips(config)
            ... ship_to_ships_client.get(
            ...     zones=["Zeebrugge"],
            ...     columns=["load_vessel_name","load_vessel_imo","discharge_vessel_name","discharge_vessel_imo","flow_quantity_cms","zone_name","start","end"]
            ... )

            .. csv-table::
                :header: "load_vessel_name","load_vessel_imo","discharge_vessel_name","discharge_vessel_imo","zone_name","start","end"

                "Red Opal","9381512","Team Falcon","9396012","Zeebrugge","2017-06-02 18:06:00","2017-06-02 18:06:00"
                "Adebomi 3","9210907","Dong-A Maia","9749544","Zeebrugge","2017-04-26 01:02:00","2017-04-26 05:05:00"
                "Lafayette Bay","9717785","Sloman Themis","9306677","Zeebrugge","2017-02-24 02:00:00","2017-02-24 04:00:00"
                "Talara","9569994","Lafayette Bay","9717785","Zeebrugge","2017-02-23 22:31:00","2017-02-23 23:00:00"
                "Hafnia Hope","9360415","Nave Bellatrix","9459084","Zeebrugge","2017-01-12 22:01:00","2017-01-12 23:01:00"
                "...","...","...","...","...","...","...","..."

        """

        query_parameters = {
            "size": size,
            "vessels": process_list_parameter(vessels),
            "zones": process_list_parameter(zones),
            "products": process_list_parameter(products),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "columns": process_list_parameter(columns),
            "withFreightView": process_bool_parameter(with_freight_view),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import Any, Dict, List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.enums import VesselTypesCPP, VesselTypesOil
from kpler.sdk.exceptions import WithProductEstimationPlatformException
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class Fixtures(KplerClient):
    """
    The ``Fixtures`` query returns the Fixtures data.
    """

    RESOURCE_NAME = "fixtures"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint fixtures.

        Examples:
            >>> from kpler.sdk.resources.fixtures import Fixtures
            ...   fixtures_client = Fixtures(config)
            ...   fixtures_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "reportedDate","Reported date","None","False","date yyyy-MM-dd"
                "vesselName","Vessel","None","False","string"
                "vesselImo","IMO","None","False","long"
                "productQuantityInTons","Quantity (t)","None","False","double"
                "vesselDeadweight","Deadweight (t)","None","False","long"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        size: Optional[int] = None,
        reported_date_after: Optional[date] = None,
        reported_date_before: Optional[date]  = None,
        lay_can_start_after: Optional[date] = None,
        lay_can_start_before: Optional[date]  = None,
        from_zones: Optional[List[str]] = None,
        to_zones: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        dwt_min: Optional[int] = None,
        dwt_max: Optional[int] = None,
        capacity_min: Optional[int] = None,
        capacity_max: Optional[int] = None,
        statuses: Optional[List[Enum]] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        vessel_types: Optional[List[str]] = None,
        vessels: Optional[List[str]] = None,
        columns: Optional[List[str]] = None,
    ) -> DataFrame:
        """

        Args:
            size: Optional[int] By default: size=1000 Maximum number of results returned.
            reported_date_after:Optional[date] Get fixtures reported on or after this date (YYYY-MM-DD)
            reported_date_before: Optional[date] Get fixtures reported on or before this date (YYYY-MM-DD)
            lay_can_start_after: Optional[date] Get fixtures with loading window start date after this date (YYYY-MM-DD)
            lay_can_start_before: Optional[date] Get fixtures with loading window start date before this date (YYYY-MM-DD)
            from_zones: Optional[List[str]] Names of the origin zones (port/region/country/continent)
            to_zones: Optional[List[str]] Names of the destination zones (port/region/country/continent)
            products: Optional[List[str]] Names of products
            dwt_min: Optional[int] Get vessels with deadweight greater or equal to this value by default 0
            dwt_max: Optional[int] Get vessels with deadweight lower or equal to this value by default 606550
            capacity_min: Optional[int] Get vessels with capacity greater or equal to this value by default 0
            capacity_max: Optional[int] Get vessels with capacity lower or equal to this value by default 606550
            statuses: Optional[List[Enum]] ```FixturesStatuses``` Status of the fixture (On Subs/In Progress/Fully Fixed/Finished/Failed/Cancelled)
            vessel_types_cpp: Optional[List[Enum]] ```VesselTypesCPP``` For given cpp vessel types (LR2, VLCC, LR3, MR, GP, LR1), only available for Liquids
            vessel_types_oil: Optional[List[Enum]] ```VesselTypesOil``` For given oil vessel types (Aframax, VLCC, Product Tanker, Suezmax, Panamax, ULCC), only available for Liquids
            vessel_types: Optional[List[str]] For other vessel types
            columns: Optional[List[str]] Retrieve all available columns when set to "all"
            vessels: Optional[List[str]] Names/IMO's of vessels
        Examples:
            >>> from datetime import date, timedelta, datetime
            ... from kpler.sdk.resources.fixtures import Fixtures
            ... fixtures_client = Fixtures(config)
            ... fixtures = fixtures_client.get(
            ... vessel_types_cpp = [VesselTypesCPP.MR],
            ... statuses = [FixturesStatuses.InProgress],
            ... columns=[
            ...         'reportedDate',
            ...         'vesselName',
            ...         'productName',
            ...         'origin',
            ...         'destination',
            ...         'status',
            ...         'vesselTypeCpp',
            ... ]
            ... )

            .. csv-table::
                :header: "Reported date","Vessel","Product","Origin","Destination","Status","Vessel Type CPP"

                "2022-10-27","Baltic Advance","ULSD","Sidi Kerir ","Koper","In Progress","MR"
                "2022-10-27","Libera","Gasoline","Burgas","Libya","In Progress","MR"
                "2022-10-27","Maria M","Gasoline","Petromidia","Corinth","In Progress","MR"
                "2022-10-27","Magnifica","Gasoline","Burgas","Koper","In Progress","MR"
                "2022-10-27","Janine K","Naphtha","Tuapse","Skhira","In Progress","MR"
                "...","...","...","...","...","...","..."
        """

        query_parameters: Dict[str, Optional[Any]] = {
            "size": size,
            "reportedDateAfter": process_date_parameter(reported_date_after),
            "reportedDateBefore": process_date_parameter(reported_date_before),            
            "layCanStartAfter": process_date_parameter(lay_can_start_after),
            "layCanStartBefore": process_date_parameter(lay_can_start_before),
            "fromZones": process_list_parameter(from_zones),
            "toZones": process_list_parameter(to_zones),
            "products": process_list_parameter(products),
            "dwtMin": dwt_min,
            "dwtMax": dwt_max,
            "capacityMin": capacity_min,
            "capacityMax": capacity_max,
            "statuses": process_enum_parameters(statuses),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "vesselTypes": process_list_parameter(vessel_types),
            "vessels": process_list_parameter(vessels),
            "columns": process_list_parameter(columns),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)




from datetime import date
from enum import Enum
from typing import List, Optional

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.exceptions import MaximumDaysPeriodExceededException
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class FleetMetricsVessels(KplerClient):

    """
    The ``FleetMetricsVessels`` endpoint provides the list of vessels with their cargo per day on a given period,
    for the Floating Storage or the Loaded Vessels metrics, and their location.
    """

    RESOURCE_NAME = "fleet-metrics/vessels"

    AVAILABLE_PLATFORMS = [Platform.Liquids, Platform.LPG, Platform.LNG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        zones: Optional[List[str]] = None,
        metric: Enum = None,
        floating_storage_duration_min: Optional[str] = None,
        floating_storage_duration_max: Optional[str] = None,
        products: Optional[List[str]] = None,
        unit: Optional[Enum] = None,
        with_freight_view: bool = False,
        vessel_types: Optional[List[Enum]] = None,
        vessel_types_alt: Optional[List[Enum]] = None,
    ):

        """

        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            zones: Optional[str] Names of countries/geographical zones
            metric: Enum ``FleetMetricsVesselsAlgo``
            floating_storage_duration_min: Optional[str] Minimum floating days [7-10-12-15-20-30-90] or [1-3-5-7-10-12-15-20-30-90] on LNG platform
            floating_storage_duration_max: Optional[str] Maximum floating days [7-10-12-15-20-30-90-Inf] or [1-3-5-7-10-12-15-20-30-90-Inf] on LNG platform
            products: Optional[str] Names of products. Not available on LNG platform
            unit: Optional[Enum] ``FleetMetricsVesselsMeasurementUnit`` by default return in bbl unit or cm on LNG platform
            with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.
            vessel_types: Optional[List[Enum]] = Vessel types, pick types based on platform.
            vessel_types_alt: Optional[List[Enum]] = For oil vessel types, only available for Liquids

        Examples:

            >>> from datetime import date
            ... from kpler.sdk.resources.fleet_metrics_vessels import FleetMetricsVessels
            ... from kpler.sdk import FleetMetricsVesselsAlgo,FleetMetricsVesselsMeasurementUnit
            ... fleet_metrics_vessels_client=FleetMetricsVessels(config)
            ... fleet_metrics_vessels_client.get(
            ...     metric=FleetMetricsVesselsAlgo.LoadedVessels,
            ...     zones=["China"],
            ...     start_date=date(2020,10,1),
            ...     end_date=date(2020,10,31),
            ...     floating_storage_duration_min="7",
            ...     floating_storage_duration_max="10",
            ...     products=["Dirty"],
            ...     unit=FleetMetricsVesselsMeasurementUnit.BBL
            ... )

            .. csv-table::
                :header:  "Date","IMO","Name","Dead Weight Tonnage","Quantity (bbl)","Family","Group","Product","Grade","Current Continent","..."

                "2020-10-01","9376749","Advantage Value","297557","1057204","Dirty","crude/co","crude","Iracema","Asia","..."
                "2020-10-01","9430313","Afra Royal","115948","530973","Dirty","crude/co","crude","Hamaca","Asia","..."
                "2020-10-01","9759836","Agios Sostis I","299983","2000003","Dirty","crude/co","crude","Arab Hy.","Asia","..."
                "2020-10-01","9653410","Al Derwazah","316884","2000003","Dirty","crude/co","crude","Kuwait","Asia","..."
                "2020-10-01","9329708","Al Jabriyah Ii","317570","2000003","Dirty","crude/co","crude","Kuwait","Asia","..."
                "...","...","...","...","...","...","...","...","...","...","..."
        """

        self.start_date = start_date
        self.end_date = end_date

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "zones": process_list_parameter(zones),
            "metric": process_enum_parameter(metric),
            "floatingStorageDurationMin": floating_storage_duration_min,
            "floatingStorageDurationMax": floating_storage_duration_max,
            "products": process_list_parameter(products),
            "unit": process_enum_parameter(unit),
            "withFreightView": process_bool_parameter(with_freight_view),
            "vesselTypes": process_enum_parameters(vessel_types, to_lower_case=False),
            "vesselTypesAlt": process_enum_parameters(vessel_types_alt, to_lower_case=False),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)

    def validate(self):
        if self.end_date and self.start_date:
            delta = (self.end_date - self.start_date).days  # type: ignore
            if delta >= 31:
                raise MaximumDaysPeriodExceededException()



from datetime import date
from enum import Enum
from typing import Any, Dict, List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class TradesSnapshot(KplerClient):
    """
    The ``TradesSnapshot`` query returns a snapshot of our trades data at a specified time.
    """

    RESOURCE_NAME = "trades/snapshot"
    COLUMNS_RESOURCE_NAME = "trades"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint trades.

        Examples:
            >>> from kpler.sdk.resources.trades_snapshot import TradesSnapshot
            ...   trades_snapshot_client = TradesSnapshot(config)
            ...   trades_snapshot_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "vessel_name","Vessel","Name of the vessel","False","string"
                "start","Date (origin)","Departure date of the vessel","False","datetime yyyy-MM-dd HH:mm"
                "origin_location_name","Origin","Origin location of the cargo","False","string"
                "origin_eta_source","Eta source (origin)","Source of the Estimated Time of Arrival to the Installation of Origin information (Port, Analyst, etc.)","False","string"
                "cargo_origin_cubic_meters","Volume (origin m3)","None","False","long"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.COLUMNS_RESOURCE_NAME)

    def get(
        self,
        snapshot_date: Optional[date] = None,
        vessels: Optional[List[str]] = None,
        from_installations: Optional[List[str]] = None,
        to_installations: Optional[List[str]] = None,
        from_zones: Optional[List[str]] = None,
        to_zones: Optional[List[str]] = None,
        buyers: Optional[List[str]] = None,
        sellers: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        trade_status: Optional[List[Enum]] = None,
        columns: Optional[List[str]] = None,
    ) -> DataFrame:
        """

        Args:
            snapshot_date: Optional[date] Date of the snapshot, if empty it returns the most recent snapshot
            vessels: Optional[List[str]] Names/IMO's of vessels
            from_installations: Optional[List[str]] Names of the origin installations
            to_installations: Optional[List[str]] Names of the destination installations (terminal/refinery)
            from_zones: Optional[List[str]] Names of the origin zones (port/region/country/continent)
            to_zones: Optional[List[str]] Names of the destination zones (port/region/country/continent)
            buyers: Optional[List[str]] Buyers of the cargo
            sellers: Optional[List[str]] Sellers of the cargo
            products: Optional[List[str]] Names of products
            start_date: Optional[date] Start of the period
            end_date: Optional[date] End of the period
            trade_status: Optional[List[Enum]] ``TradesStatus`` Return only trades of a particular status. By default value is scheduled.
            columns: Optional[List[str]] Retrieve all available columns when set to "all"
        Examples:
            >>> from datetime import date, timedelta, datetime
            ... from kpler.sdk.resources.trades_snapshot import TradesSnapshot
            ... trades_snapshot_client = TradesSnapshot(config)
            ... snapshots = trades_snapshot_client.get(
            ... snapshot_date = datetime.strptime('2022/01/01', '%Y/%m/%d'),
            ... to_zones = ["France"],
            ... products = ["crude"],
            ... start_date = datetime.strptime('2021/01/01', '%Y/%m/%d'),
            ... columns=[
            ...         "vessel_name",
            ...         "origin_country_name",
            ...         "origin_location_name",
            ...         "destination_location_name",
            ...         "start",
            ...         "end"
            ... ]
            ... )
            >>> print(snapshots.headers['snapshotName'])
            "2022-01-01T000100"
            >>> print(snapshots.headers['lastAvailableDate'])
            "2022-10-19T034600"

            .. csv-table::
                :header: "vessel_name", "origin_country_name", "origin_location_name","destination_location_name","start","end","trade_id","product_id"

                "Sakura Princess", "Russian Federation", "Novorossiysk", "Fos", "2022-02-05 18:48:00", "2022-02-18 23:36:00", "13163188", "1360"
                "Alfa Baltica", "United Kingdom", "Hound Point", "Le Havre", "2022-01-29 08:31:00", "2022-02-09 22:25:00", "13214389", "1502"
                "Scf Baikal", "Equatorial Guinea", "Zafiro FPSO", "Fos", "2022-01-27 14:18:00", "2022-02-20 02:17:00", "13263935", "2430"
                "Calida", "Russian Federation", "Umba FSO", "Le Havre", "2022-01-25 21:59:00", "2022-02-02 11:00:00", "13107517", "1940"
                "Siri Knutsen", "United Kingdom", "Culzean", "Le Havre", "2022-01-25 18:50:00", "2022-01-27 17:01:00", "13208418", "2488"
                "...", "...", "...", "...", "...", "...", "...", "..."

        """

        query_parameters: Dict[str, Optional[Any]] = {
            "snapshotDate": process_date_parameter(snapshot_date),
            "vessels": process_list_parameter(vessels),
            "fromInstallations": process_list_parameter(from_installations),
            "toInstallations": process_list_parameter(to_installations),
            "fromZones": process_list_parameter(from_zones),
            "toZones": process_list_parameter(to_zones),
            "buyers": process_list_parameter(buyers),
            "sellers": process_list_parameter(sellers),
            "products": process_list_parameter(products),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "tradeStatus": process_enum_parameters(trade_status),
            "columns": process_list_parameter(columns),
        }
        header_parameters: List[str] = ["snapshotName", "lastAvailableDate"]
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters, header_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class FleetUtilizationSeries(KplerClient):

    """
    The ``FleetUtilizationSeries`` endpoint returns current and historical supply & demand capacity balance, total,loaded & ballast capacity evolution and capacity available by products.
    """

    RESOURCE_NAME = "fleet-utilization/series"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        zones: Optional[List[str]] = None,
        metric: Optional[Enum] = None,
        period: Optional[Enum] = None,
        split: Optional[Enum] = None,
        products: Optional[List[str]] = None,
        unit: Optional[Enum] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        vessel_states: Optional[List[Enum]] = None,
        percent: Optional[bool] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        vessel_types: Optional[List[Enum]] = None,
        previous_zones: Optional[List[str]] = None,
        next_zones: Optional[List[str]] = None,
        vessel_directions: Optional[List[str]] = None,
        distinct_aggregation: bool = False,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            zones: Optional[List[str]] Names of countries/geographical zones
            metric: Optional[Enum] ``FleetUtilizationSeriesMetric``
            period: Optional[Enum] ``FleetUtilizationSeriesPeriod``
            split: Optional[Enum] ``FleetUtilizationSeriesSplit``
            products: Optional[List[str]] Names of products
            unit: Optional[Enum] ``FleetUtilizationSeriesUnit``
            vessel_types_cpp: Optional[List[Enum]] ``VesselTypesCPP``
            vessel_types_oil: Optional[List[Enum]] ``VesselTypesOil``
            vessel_states: Optional[List[Enum]] ``FleetUtilizationSeriesVesselsState``
            percent: Optional[bool] Access percentage of loaded vessels over ballast vessels
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            vessel_types: Optional[List[Enum]] ``VesselTypesDry`` ``VesselTypesLNG`` ``VesselTypesLPG``
            previous_zones: Optional[List[str]] Names of zones (port/region/country/continent) the vessel is coming from
            next_zones: Optional[List[str]] Names of zones (port/region/country/continent) the vessel is going to
            vessel_directions: Optional[List[str]] Names of the vessel’s course of direction on the given day
            distinct_aggregation: Option[bool] Show or hide distinct vessels by default false

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.fleet_utilization_series import FleetUtilizationSeries
            ... from kpler.sdk import FleetUtilizationSeriesMetric, FleetUtilizationSeriesPeriod,FleetUtilizationSeriesSplit, FleetUtilizationSeriesUnit
            ... fleet_utilization_series_client = FleetUtilizationSeries(config)
            ... fleet_utilization_series_client.get(
            ...     start_date=date(2020,10,1),
            ...     end_date=date(2020,11,1),
            ...     zones=["Japan"],
            ...     metric=FleetUtilizationSeriesMetric.Count,
            ...     period=FleetUtilizationSeriesPeriod.Daily,
            ...     split=FleetUtilizationSeriesSplit.Product,
            ...     unit=FleetUtilizationSeriesUnit.MT
            ... )

            .. csv-table::
                :header:  "Date","Clean Products","crude/co","DPP","NPC","Other"

                "2020-10-01","114","62","31","37","13"
                "2020-10-02","114","60","36","40","14"
                "2020-10-03","104","59","38","43","12"
                "2020-10-04","105","55","38","42","13"
                "2020-10-05","109","60","40","42","13"
                "2020-10-06","100","61","37","36","14"
                "2020-10-07","103","52","31","30","14"
                "2020-10-08","105","38","33","24","13"
                "2020-10-09","109","46","31","17","11"
                "2020-10-10","115","48","35","21","10"
                "...","...","...","...","...","...",
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "zones": process_list_parameter(zones),
            "metric": process_enum_parameter(metric),
            "period": process_enum_parameter(period),
            "split": process_enum_parameter(split),
            "products": process_list_parameter(products),
            "unit": process_enum_parameter(unit),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "vesselStates": process_enum_parameters(vessel_states, False),
            "percent": process_bool_parameter(percent),
            "gte": gte,
            "lte": lte,
            "vesselTypes": process_enum_parameters(vessel_types, False),
            "previousZones": process_list_parameter(previous_zones),
            "nextZones": process_list_parameter(next_zones),
            "vesselDirections": process_list_parameter(vessel_directions),
            "distinctAggregation": process_bool_parameter(distinct_aggregation),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



import json
import logging
from typing import Any, Dict, List, Optional

import pandas as pd
from pkg_resources import get_distribution
import requests
from requests.auth import HTTPBasicAuth

from kpler.sdk.configuration import Configuration
from kpler.sdk.exceptions import (
    AuthenticationClassError,
    AuthenticationError,
    HttpError,
    RestrictedPlatformException,
)
from kpler.sdk.helpers import bytes_to_pandas_data_frame, filter_nones_from_dict


VERSION = get_distribution("kpler.sdk").version


class KplerClient:
    """
    Base class for resources.

    Attributes:
        platform(Platform): An enum of type Platform to indicate the Platform to connect to
        email(str): The email to be used for login
        password(str): The password associated to the email

    """

    def __init__(
        self,
        configuration: Configuration,
        available_platforms: List,
        column_ids: bool = True,
        log_level=None,
    ):
        """
        Initiate the connection to Kpler servers.

        We do the login once and we keep the token.
        But we need to make sure we renew it before it expires.

        Args:
            configuration (Configuration): Configuration instance object which contains all the details (platform, login, password)
        """

        if configuration.platform not in available_platforms:
            raise RestrictedPlatformException(available_platforms)

        self.available_platforms = available_platforms

        logging.basicConfig(format="%(asctime)s [%(levelname)s] %(message)s")
        self.configuration = configuration
        self.column_ids = column_ids
        self.logger = self.configuration.logger

        self._get_mapping()

    def _get_mapping(self):
        """
        Get the type mapping for this endpoint. It will be used to map type in Pandas.
        """
        if not self.configuration.auth_validated:
            self.mapping = {}
            return

        try:
            cols = self.get_columns()
            self.mapping = cols.set_index("id")["type"].to_dict()
            self.mapping.update(cols.set_index("name")["type"].to_dict())
        except Exception:
            # the endpoint does not have it, or anything else
            self.mapping = {}

    def _get_raw(self, resource: str, params: Dict[str, Optional[Any]] = {}):
        """
        Internal use only!

        This is the centralized method to query the server.
        """
        if not self.configuration.auth_validated:
            raise AuthenticationClassError()

        url = f"{self.configuration.base_url}/{resource}"
        params = params or {}
        params = filter_nones_from_dict(params)
        url_pretty = url + "?" + "&".join([f"{k}={v}" for k, v in params.items()])
        self.logger.debug(f"Requesting {url_pretty}")

        headers = {"X-SDK-Version": f"python-{VERSION}"}
        if self.column_ids:
            headers["X-use-columns-ids-headers"] = ""

        response = requests.get(
            url,
            params=params,  # type: ignore
            auth=HTTPBasicAuth(self.configuration.email, self.configuration.password),
            stream=True,
            headers=headers,
            proxies=self.configuration.proxies,
            cert=self.configuration.certificates,
            verify=self.configuration.verify,
        )
        if response.status_code == 200:
            return response
        elif response.status_code in (401, 403):
            raise AuthenticationError(response.content)
        else:
            raise HttpError(response.status_code, response.content)

    def _get_dataframe(
        self, resource: str, params: Dict[str, Optional[Any]], header_parameters=None
    ) -> pd.DataFrame:

        if header_parameters is None:
            header_parameters = []
        if self.configuration.platform not in self.available_platforms:
            raise RestrictedPlatformException(self.available_platforms)

        self.validate()

        response = self._get_raw(resource, params)
        content: bytes = response.content

        try:
            if len(header_parameters) == 0:
                return bytes_to_pandas_data_frame(content, self.mapping)
            else:
                df = bytes_to_pandas_data_frame(content, self.mapping)
                df._metadata = ["headers"]
                df.headers = {
                    header_param: response.headers[header_param]
                    for header_param in header_parameters
                }

                return df
        except AuthenticationClassError:
            self.logger.error("You are not authenticated. Please execute 'Configuration' first.")
        except AuthenticationError as e:
            self.logger.error(f"Authentication error on request: returned [{e}]")
        except Exception as e:
            self.logger.error(f"Error on request: returned [{e}]")

    def _get_columns_for_resource(self, resource: str, columns_path: str = "columns") -> pd.DataFrame:
        try:
            content: bytes = self._get_raw(f"{resource}/{columns_path}").content
            data: dict = json.loads(content)
            df = pd.DataFrame(data["selected"] + data["unselected"])
            if "shortId" in df.columns:
                df.drop("shortId", axis=1, inplace=True)
            if "columnName" in df.columns:
                df.rename(columns={"columnName": "name"}, inplace=True)
            df.columns = df.columns.str.strip()
            return df
        except AuthenticationClassError:
            self.logger.error("You are not authenticated. Please execute 'Configuration' first.")
        except AuthenticationError as e:
            self.logger.error(f"Authentication error on request: returned [{e}]")
        except Exception as e:
            self.logger.error(f"Error on request: returned [{e}]")
        return pd.DataFrame()

    def _search(self, query_parameters: Dict[str, str]) -> pd.DataFrame:
        if not self.configuration.auth_validated:
            self.logger.error("You are not authenticated. Please execute 'Configuration' first.")
            return pd.DataFrame()

        try:
            result = requests.get(
                f"{self.configuration.base_url}/search",
                params=query_parameters,  # type: ignore
                auth=HTTPBasicAuth(self.configuration.email, self.configuration.password),
                stream=True,
                headers={"X-SDK-Version": f"python-{VERSION}"},
                proxies=self.configuration.proxies,
                cert=self.configuration.certificates,
                verify=self.configuration.verify,
            )
            return pd.DataFrame.from_dict(result.json())
        except Exception as e:
            self.logger.error(f"Error on search request: returned [{e}]")
        return pd.DataFrame()

    def get_columns(self):
        raise NotImplementedError

    def get(self):
        raise NotImplementedError

    def search(self, q: str) -> Any:
        raise AttributeError(f"search function is not defined for {self.__class__.__name__}")

    def validate(self):
        return True



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.enums import MarginsGranularity, MarginsSplit
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameter,
    process_list_parameter,
)


class Margins(KplerClient):
    """
   The Margins endpoint returns the margins for a point of interest (installation/zone) on a daily, weekly,
   weekly EIA (for US), monthly and yearly basis.
    """

    RESOURCE_NAME = "refineries/margins"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint refineries margins.


        Examples:
            >>> from kpler.sdk.resources.refineries.margins import Margins
            ... margins_client = Margins(config)
            ... margins_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date (timestamp)","Date, within the start_date and end_date. Data is provided with ascending order on date. Format YYYY-MM-DD.","False","string"
                "zones","Zones","List of zones specified in the parameter zones.","False","list of string"
                "installations","Installations","List of installations specified in the parameter installations.","False","list of string"
                "splitValue","Split Value","Name of the bucket corresponding to the specified split. EG Americas or Asia for split by Continent.","False","string"
                "...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
            self,
            players: Optional[List[str]] = None,
            installations: Optional[List[str]] = None,
            zones: Optional[List[str]] = None,
            start_date: Optional[date] = None,
            end_date: Optional[date] = None,
            granularity: Optional[Enum] = None,
            split: Optional[Enum] = None,

    ) -> DataFrame:
        """
        Args:
            players:  Optional[str] Names of players
            installations: Optional[List[str]] Names of installations
            zones: Optional[List[str]] Names of countries/geographical zones
            start_date:  Optional[date] Start of the period (YYYY-MM-DD), must be after 2017-01-01
            end_date:  Optional[date] End of the period (YYYY-MM-DD), maximum of 7 days from today
            granularity: Optional[Enum] ``MarginsGranularity``
            split: Optional[Enum] ``MarginsSplit``

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.refineries.margins import Margins
            ... from kpler.sdk import MarginsGranularity, MarginsSplit
            ... margins_client = Margins(config)
            ... margins_client.get(
            ...     installations=["xxxx"],
            ...     zones=["United States"],
            ...     start_date=date(2023, 4, 1),
            ...     end_date=date(2023, 7, 31),
            ...     player=["xxx"],
            ...     granularity=MarginsGranularity.Monthly,
            ...     split=MarginsSplit.Total,
            ... )

            .. csv-table::
                :header: "Date","Zones","Installations","Split Value","Value","Unit"

                "2023-04-01","United States",,"Total","12.0","$/bbl"
                "2023-05-01","United States",,"Total","13.5","$/bbl"
                "2023-06-01","United States",,"Total","21.0","$/bbl"
                "2023-07-01","United States",,"Total","23.4","$/bbl"
        """

        query_parameters = {
            "players": process_list_parameter(players),
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "split": process_enum_parameter(split, to_lower_case=False),
            "granularity": process_enum_parameter(granularity, to_lower_case=False),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class BallastCapacitySeries(KplerClient):

    """
    The ``BallastCapacitySeries`` endpoint returns daily/ weekly/ monthly aggregated data values for a given zone/ vessel type/ time period /last product carried.
    """

    RESOURCE_NAME = "ballast-capacity/series"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        zones: Optional[List[str]] = None,
        metric: Optional[Enum] = None,
        period: Optional[Enum] = None,
        split: Optional[Enum] = None,
        products: Optional[List[str]] = None,
        unit: Optional[Enum] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        vessel_types: Optional[List[Enum]] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        with_freight_view: bool = False,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            zones: Optional[List[str]] Names of countries/geographical zones
            metric: Optional[Enum] ``BallastCapacitySeriesMetric``
            period: Optional[Enum] ``BallastCapacitySeriesPeriod``
            split: Optional[Enum] ``BallastCapacitySeriesSplit``
            products: Optional[List[str]] Names of products
            unit: Optional[Enum] ``BallastCapacitySeriesUnit``
            vessel_types_cpp: Optional[List[Enum]] ``VesselTypesCPP``
            vessel_types_oil: Optional[List[Enum]] ``VesselTypesOil``
            vessel_types: Optional[List[Enum]] ``VesselTypesDry`` ``VesselTypesLNG`` ``VesselTypesLPG``
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.

        Examples:
            >>> from kpler.sdk.resources.ballast_capacity_series import BallastCapacitySeries
            ... from kpler.sdk import BallastCapacitySeriesMetric, BallastCapacitySeriesPeriod, BallastCapacitySeriesSplit
            ... ballast_capacity_series_client = BallastCapacitySeries(config)
            ... ballast_capacity_series_client.get(
            ...     metric=BallastCapacitySeriesMetric.Capacity,
            ...     period=BallastCapacitySeriesPeriod.Daily,
            ...     split=BallastCapacitySeriesSplit.VesselType,
            ... )

            .. csv-table::
                :header:  "Date","MR","GP","LR3","LR2","VLCC","LR1"

                "2020-11-22","10724.0","NaN","NaN","NaN","NaN","NaN"
                "2020-11-23","616555.0","2080981.0","1194773.0","1004319.0","163657.0","679003.0"
                "2020-11-24","1287340.0","2944327.0","4162769.0","2826019.0","1340769.0","2647163.0"
                "2020-11-25","854538.0","2975890.0","3930704.0","3186276.0","1226344.0","5675351.0"
                "2020-11-26","803699.0","2393202.0","2431821.0","2325491.0","571266.0","2991957.0"
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "zones": process_list_parameter(zones),
            "metric": process_enum_parameter(metric),
            "period": process_enum_parameter(period),
            "split": process_enum_parameter(split),
            "products": process_list_parameter(products),
            "unit": process_enum_parameter(unit),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "vesselTypes": process_enum_parameters(vessel_types, False),
            "gte": gte,
            "lte": lte,
            "withFreightView": process_bool_parameter(with_freight_view),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_date_parameter,
    process_list_parameter,
)


class Overview(KplerClient):
    """
    The Overview endpoint returns a list of refineries along
    with their respective characteristics for a point of interest (installation/zone).
    """

    RESOURCE_NAME = "refineries/overview"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint refineries overview.

        Examples:
            >>> from kpler.sdk.resources.refineries.overview import Overview
            ... overview_client = Overview(config)
            ... overview_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date (timestamp)","Date, within the start_date and end_date. Data is provided with ascending order on date. Format YYYY-MM-DD.","False","string"
                "zones","Zones","List of zones specified in the parameter zones.","False","list of string"
                "installations","Installations","List of installations specified in the parameter installations.","False","list of string"
                "refinery","Refinery","Name of the refinery","False","string"
                "owner","Owner","Name of the owner","False","string"
                "...","...","...","...","...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
            self,
            players: Optional[List[str]] = None,
            installations: Optional[List[str]] = None,
            zones: Optional[List[str]] = None,
            start_date: Optional[date] = None,
            end_date: Optional[date] = None,
    ) -> DataFrame:
        """
        Args:
            players:  Optional[str] Names of players
            installations: Optional[List[str]] Names of installations
            zones: Optional[List[str]] Names of countries/geographical zones
            start_date:  Optional[date] Start of the period (YYYY-MM-DD), must be after 2017-01-01
            end_date:  Optional[date] End of the period (YYYY-MM-DD), maximum of 7 days from today

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.refineries.overview import Overview
            ... overview_client = Overview(config)
            ... overview_client.get(
            ...     installations=["xxxx"],
            ...     zones=["United States"],
            ...     start_date=date(2023, 4, 1),
            ...     end_date=date(2023, 7, 31),
            ...     player=["xxx"],
            ... )

            .. csv-table::
                :header: "Start Date","End Date","Zones","Installations","Refinery","Owner","Age (years)","NCI","Type","..."


                "2023-04-01","2023-07-01","Tacoma","Targa Tacoma","Seaport Sound Refinery","Targa Resources","46","None","Other","..."
                "2023-04-01","2023-07-01","Lovington","Lovington Refinery","Lovington Refinery","HollyFrontier","2","10.3","Complex","..."
                "2023-04-01","2023-07-01","Indianapolis","Indianapolis Refinery","Indianapolis Refinery","Marathon Oil Corporation","63","None","Other","..."
         """

        query_parameters = {
            "players": players,
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_bool_parameter,
    process_date_parameter,
    process_enum_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class FleetUtilizationVessels(KplerClient):

    """
    The ``FleetUtilizationVessels`` provides current and historical supply & demand capacity balance, total,loaded & ballast capacity evolution and capacity available by products.
    """

    RESOURCE_NAME = "fleet-utilization/vessels"
    COLUMNS_RESOURCE_NAME = "fleet-utilization"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint fleet-utilization.

        Examples:
            >>> from kpler.sdk.resources.fleet_utilization_vessels import FleetUtilizationVessels
            ... fleet_utilization_vessels_client=FleetUtilizationVessels(config)
            ... fleet_utilization_vessels_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date (timestamp)","","False","date yyyy-MM-dd"
                "vessel_imo","IMO","IMO of the vessel","False","long"
                "vessel","Name","Name of the vessel","False","string"
                "vessel_dwt_tons","Dead Weight Tonnage","Dead Weight Tonnage","False","long"
                "vessel_type","Vessel Type","Type of the vessel","False","string"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.COLUMNS_RESOURCE_NAME)

    def get(
        self,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        zones: Optional[List[str]] = None,
        products: Optional[List[str]] = None,
        vessel_types_cpp: Optional[List[Enum]] = None,
        vessel_types_oil: Optional[List[Enum]] = None,
        vessel_states: Optional[List[Enum]] = None,
        unit: Optional[Enum] = None,
        size: Optional[int] = None,
        gte: Optional[int] = None,
        lte: Optional[int] = None,
        vessel_types: Optional[List[Enum]] = None,
        period: Optional[Enum] = None,
        previous_zones: Optional[List[str]] = None,
        next_zones: Optional[List[str]] = None,
        vessel_directions: Optional[List[str]] = None,
        distinct_aggregation: bool = False,
        vessels: Optional[List[str]] = None,
    ):

        """
        Args:
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            zones: Optional[List[str]] Names of countries/geographical zones
            products: Optional[List[str]] Names of products
            vessel_types_cpp: Optional[List[Enum]] ``VesselTypesCPP``
            vessel_types_oil: Optional[List[Enum]] ``VesselTypesOil``
            vessel_states: Optional[List[Enum]] ``FleetUtilizationVesselsState``
            unit: Optional[Enum] ``FleetUtilizationVesselsUnit``
            size: Optional[int] Maximum number of fleet utilization vessels returned
            gte: Optional[int] Get vessels with deadweight/capacity greater or equals to this value by default 0
            lte: Optional[int] Get vessels with deadweight/capacity lower or equals to this value by default 606550
            vessel_types: Optional[List[Enum]] ``VesselTypesDry`` ``VesselTypesLNG`` ``VesselTypesLPG``
            period: Optional[Enum] ``FleetUtilizationVesselsPeriod``
            previous_zones: Optional[List[str]] Names of zones (port/region/country/continent) the vessel is coming from
            next_zones: Optional[List[str]] Names of zones (port/region/country/continent) the vessel is going to
            vessel_directions: Optional[List[str]] Names of the vessel’s course of direction on the given day
            distinct_aggregation: Option[bool] Show or hide distinct vessels by default false
            vessels: Optional[List[str]] Names or IMOs of the vessels

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.fleet_utilization_vessels import FleetUtilizationVessels
            ... from kpler.sdk import FleetUtilizationVesselsState
            ... fleet_utilization_vessels_client=FleetUtilizationVessels(config)
            ... fleet_utilization_vessels_client.get(
            ...     start_date=date(2020,10,1),
            ...     end_date=date(2020,11,1),
            ...     zones=["Japan"],
            ...     products=["gasoline", "DPP"],
            ...    vessel_states=[FleetUtilizationVesselsState.Loaded],
            ...     size=5
            ... )

            .. csv-table::
                :header:  "Date (timestamp)","IMO","Name","Dead Weight Tonnage","Vessel Type","Vessel State","Current Zone","Last Product on board"

                "2020-10-01","9751121","Kakusho Iii","3500","GP","Loaded","Japan","DPP"
                "2020-10-01","9774410","Sun Venus","6879","GP","Loaded","Japan","DPP"
                "2020-10-01","9524475","Suez Rajan","158574","LR3","Loaded","Japan","DPP"
                "2020-10-01","9636644","Maersk Aegean","37538","MR","Loaded","Japan","DPP"
                "2020-10-01","9346873","Mare Oriens","110295","LR2","Loaded","Japan","DPP"
        """

        query_parameters = {
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "zones": process_list_parameter(zones),
            "products": process_list_parameter(products),
            "vesselTypesCpp": process_enum_parameters(vessel_types_cpp, False),
            "vesselTypesOil": process_enum_parameters(vessel_types_oil, False),
            "vesselStates": process_enum_parameters(vessel_states, False),
            "unit": process_enum_parameter(unit),
            "size": size,
            "gte": gte,
            "lte": lte,
            "vesselTypes": process_enum_parameters(vessel_types, False),
            "period": process_enum_parameter(period),
            "previousZones": process_list_parameter(previous_zones),
            "nextZones": process_list_parameter(next_zones),
            "vesselDirections": process_list_parameter(vessel_directions),
            "distinctAggregation": process_bool_parameter(distinct_aggregation),
            "vessels": process_list_parameter(vessels),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



class KplerException(Exception):
    """All errors related to making an API request extend this."""

    pass


class HttpError(KplerException):
    """Errors produced at the HTTP layer."""

    def __init__(self, status_code, body):
        self.status_code = status_code
        self.body = body

    def __repr__(self):
        return f"<HttpError, status code={self.status_code}, message={self.body}>"


class AuthenticationError(HttpError):
    """Errors due to invalid authentication credentials."""

    def __init__(self, body):
        self.status_code = 401
        self.body = body

    def __repr__(self):
        return f"<AuthenticationError, message={self.body}>"


class AuthenticationClassError(KplerException):
    """Errors due to not authenticated user."""

    def __repr__(self):
        return "<HttpError, you are not authenticated>"


class RestrictedPlatformException(KplerException):
    """Errors due to not available platform chosen."""

    def __init__(self, available_platforms):
        self.available_platforms = available_platforms

    def __str__(self):
        return repr(
            f"<RestrictedPlatformException, the platform which you have specified is not available for this client."
            f" Please choose from the following ones: {self.available_platforms}>"
        )


class MaximumDaysPeriodExceededException(KplerException):
    """Errors due to maximum days period exceeded."""

    def __str__(self):
        return repr(
            "<<MaximumDaysPeriodExceededException, period between start and end dates should be less then 31>"
        )


class WithProductEstimationPlatformException(KplerException):
    """Error due to inappropriate platform was chosen."""

    def __str__(self):
        return repr(
            "<<WithProductEstimationPlatformException, product estimation feature is for CPP trades only>"
        )



from datetime import date, datetime
import enum
from io import StringIO
import logging
from typing import Any, Dict, Iterable, List, Optional

import pandas as pd

logger = logging.getLogger(__name__)


def filter_nones_from_dict(query_parameters: Dict[str, Optional[Any]]) -> Dict[str, Any]:
    """
    This function takes a list of parameters as a dict, and returns a dict containing only non null parameters

    Examples:
        >>> filter_nones_from_dict({"key1": None, "key2": "some value"})
        {"key2": "some value"}

    Args:
        query_parameters (dict): query parameters in the form of a dict with `None` values

    Returns:
        dict: a dict with None values filtered

    """
    return {k: v for k, v in query_parameters.items() if v}


def process_list_parameter(parameter: Optional[List[str]]) -> Optional[str]:
    return convert_list_to_string_query_params(parameter) if parameter else None


def process_bool_parameter(parameter: Optional[bool]) -> Optional[str]:
    return str(parameter).lower() if parameter is not None else None


def process_date_parameter(parameter: Optional[date]) -> Optional[str]:
    return parameter.isoformat() if parameter else None


def process_updates_date_parameter(parameter: Optional[datetime]) -> Optional[str]:
    return parameter.strftime('%Y-%m-%dT%H%M%S') if parameter else None


def process_flow_snapshot_parameter(parameter: Optional[datetime]) -> Optional[str]:
    return parameter.strftime('%Y-%m-%d') if parameter else None


def process_enum_parameters(
    parameter: Optional[List[enum.Enum]], to_lower_case: bool = True
) -> Optional[str]:
    return (
        convert_list_to_string_query_params(
            [p.value.lower() if to_lower_case else p.value for p in parameter]
        )
        if parameter
        else None
    )


def process_enum_parameter(
    parameter: Optional[enum.Enum], to_lower_case: bool = True
) -> Optional[str]:
    return (
        (str(parameter.value.lower()) if to_lower_case else str(parameter.value))
        if parameter
        else None
    )


def convert_list_to_string_query_params(list_of_parameters: Iterable[str]) -> str:
    if type(list_of_parameters) in (list, tuple):
        return ",".join(list_of_parameters)
    else:
        # list_of_parameters is already string here
        # but let's cast it other the tests are failing
        return str(list_of_parameters)


def prepare_pandas_mapping(mapping: Dict[str, Any]) -> Dict[str, str]:
    """
    Will return the pandas mapping based on the kpler one returned by `columns` endpoints.
    """
    pandas_mapping: Dict[str, str] = {}

    for c, _type in mapping.items():
        pt: str = "object"
        if _type in ("integer", "long", "float", "double"):
            pt = "numeric"
        elif _type == "boolean":
            pt = "boolean"
        elif _type.startswith("date"):
            pt = "datetime"
        # else it's object (for strings)
        pandas_mapping[c] = pt
    return pandas_mapping


def bytes_to_pandas_data_frame(byte_contents: bytes, mapping: Dict[str, Any]) -> pd.DataFrame:
    """
    Will convert bytes content to a Pandas dataframe

    - If the mapping is given (not empty), then we just read the dataframe without type infering.
    We then apply the mapping coming from the get_columns endpoint.
    - If the mapping is not given, then we create the dataframe with type infering. But we try to
    set datetime types manually behind just in case.
    """
    if len(mapping) > 0:
        df = pd.read_csv(
            StringIO(byte_contents.decode("utf-8")),
            sep=";",
            dtype="object",
        )

        raw_mapping: Dict[str, str] = prepare_pandas_mapping(mapping)

        for col, _type in raw_mapping.items():
            if col not in df.columns:
                continue

            if _type == "numeric":
                # will return integer when possible, float otherwise
                df[col] = pd.to_numeric(df[col], errors="ignore")
            elif _type.startswith("datetime"):
                df[col] = pd.to_datetime(df[col], errors="ignore")
            elif _type == "boolean":
                # for boolean, since KPLER API returns "true/false", this is evaluated as a non-null value,
                # hence always TRUE for a pandas boolean. So we need to tweak that a bit.
                df[col] = df[col].str.lower() == "true"

        # now maybe the dataframe contains columns that are not returned by the `columns` endpoint
        # (e.g. for dynamic columns returned when doing a split)
        # let's try to convert them to numeric values
        for col in [col for col in df.columns if col not in raw_mapping.keys()]:
            if pd.api.types.is_string_dtype(df[col]):
                df[col] = pd.to_numeric(df[col], errors="ignore")

    else:
        df = pd.read_csv(
            StringIO(byte_contents.decode("utf-8")),
            sep=";",
            low_memory=False,
        )
        # let's convert the dates if possible
        # we have at least ETA or date in the column name
        cols_dates = [
            col
            for col in df.columns
            if not any([o.lower() in col.lower() for o in (["jsonupdates"])])
            and any([p.lower() in col.lower() for p in ("from", "until", "date", "eta")])
            and str(df[col].head(1).values).count("-")
            > 1  # Adding this to avoid converting i.e 2016-02 to 2016-02-01
        ]
        for col in cols_dates:
            if pd.api.types.is_string_dtype(df[col]):
                df[col] = pd.to_datetime(df[col], errors="ignore")

    return df



from datetime import date
from enum import Enum
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameter,
    process_list_parameter,
)


class UtilizationRates(KplerClient):
    """
    The UtilizationRates endpoint returns the utilization rate for a point of interest (installation/zone) on a daily,
    weekly, weekly EIA (for US), monthly and yearly basis.
    This is formulated as runs/operational capacity.
    """

    RESOURCE_NAME = "refineries/utilization-rates"

    AVAILABLE_PLATFORMS = [Platform.Liquids]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint
        refineries utilization-rates.

        Examples:
            >>> from kpler.sdk.resources.refineries.utilization_rates import UtilizationRates
            ... utilization_rates_client = UtilizationRates(config)
            ... utilization_rates_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "date","Date (timestamp)","Date, within the start_date and end_date. Data is provided with ascending order on date. Format YYYY-MM-DD.","False","string"
                "zones","Zones","List of zones specified in the parameter zones.","False","list of string"
                "installations","Installations","List of installations specified in the parameter installations.","False","list of string"
                "splitValue","Split Value","Name of the bucket corresponding to the specified split. EG Americas or Asia for split by Continent.","False","string"
                "metric","Metric","Corresponding to the endpoint.","False","string"
                "...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
            self,
            players: Optional[List[str]] = None,
            installations: Optional[List[str]] = None,
            zones: Optional[List[str]] = None,
            start_date: Optional[date] = None,
            end_date: Optional[date] = None,
            products: Optional[List[Enum]] = None,
            unit: Optional[Enum] = None,
            granularity: Optional[Enum] = None,
            split: Optional[Enum] = None,

    ) -> DataFrame:
        """
        Args:
            players:  Optional[str] Names of players
            installations: Optional[List[str]] Names of installations
            zones: Optional[List[str]] Names of countries/geographical zones
            start_date:  Optional[date] Start of the period (YYYY-MM-DD), must be after 2017-01-01
            end_date:  Optional[date] End of the period (YYYY-MM-DD), maximum of 7 days from today
            granularity: Optional[Enum] ``UtilizationGranularity``
            split: Optional[Enum] ``UtilizationSplit``

        Examples:
            >>> from datetime import date
            ... from kpler.sdk.resources.refineries.utilization_rates import UtilizationRates
            ... from kpler.sdk import UtilizationGranularity, UtilizationSplit
            ... utilization_rates_client = UtilizationRates(config)
            ... utilization_rates_client.get(
            ...     installations=["xxxx"],
            ...     zones=["United States"],
            ...     start_date=date(2023, 4, 1),
            ...     end_date=date(2023, 7, 31),
            ...     player=["xxx"],
            ...     granularity=UtilizationGranularity.Monthly,
            ...     split=UtilizationSplit.Total,
            ... )

            .. csv-table::
                :header: "Date","Zones","Installations","Split Value","Metric","Value","Unit"

                "2023-04-01","United States",,"Total","Utilization Rate","88.0","%"
                "2023-05-01","United States",,"Total","Utilization Rate","89.0","%"
                "2023-06-01","United States",,"Total","Utilization Rate","100.0","%"
                "2023-07-01","United States",,"Total","Utilization Rate","91.0","%"
        """

        query_parameters = {
            "players": process_list_parameter(players),
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "split": process_enum_parameter(split, to_lower_case=False),
            "granularity": process_enum_parameter(granularity, to_lower_case=False),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)


from enum import Enum


class BallastCapacityPortCallsSources(Enum):
    """"""

    Forecast = "Forecast"  #:
    Market = "Market"  #:
    AIS = "AIS"  #:
    Port = "Port"  #:
    Analyst = "Analyst"  #:
    Fixture = "Fixture"  #:


class BallastCapacityPortCallsVesselStates(Enum):
    """"""

    Ballast = "Ballast"  #:
    Loaded = "Loaded"  #:


class BallastCapacitySeriesPeriod(Enum):
    """"""

    Monthly = "monthly"  #:
    Weekly = "weekly"  #:
    Daily = "daily"  #:


class BallastCapacitySeriesSplit(Enum):
    """"""

    Total = "total"  #:
    Country = "country"  #:
    Source = "source"  #:
    VesselType = "vesselType"  #:
    VesselTypeCpp = "vesselTypeCpp"  #:
    VesselTypeOil = "vesselTypeOil"  #:


class BallastCapacitySeriesUnit(Enum):
    """"""

    MT = "mt"  #:
    KT = "kt"  #:


class BallastCapacitySeriesMetric(Enum):
    """"""

    Count = "count"  #:
    DeadWeight = "deadWeight"  #:
    Capacity = "capacity"  #:


class CongestionSeriesPeriod(Enum):
    """"""

    Annually = "years"  #:
    Monthly = "months"  #:
    Weekly = "weeks"  #:
    Quarterly = "quarters"  #:
    Daily = "days"  #:
    EIA = "eia"  #:


class CongestionVesselsPeriod(Enum):
    """"""

    Annually = "years"  #:
    Monthly = "months"  #:
    Weekly = "weeks"  #:
    Quarterly = "quarters"  #:
    Daily = "days"  #:
    EIA = "eia"  #:


class CongestionSeriesSplit(Enum):
    """"""

    Total = "total"  #:
    Port = "port"  #:
    Installation = "installation"  #:
    Country = "country"  #:
    VesselType = "vesselType"  #:
    VesselTypeCpp = "vesselTypeCpp"  #:
    VesselTypeOil = "vesselTypeOil"  #:
    VesselOperations = "VesselOperations"  #:
    WaitingStatus = "waitingStatus"  #:
    Product = "product"  #:
    Grade = "grade"  #:


class CongestionSeriesUnit(Enum):
    """"""

    MT = "mt"  #:
    KT = "kt"  #:


class CongestionSeriesMetric(Enum):
    """"""

    Count = "count"  #:
    DeadWeight = "deadWeight"  #:
    Duration = "duration"  #:
    Capacity = "capacity"  #:


class CongestionSeriesOperation(Enum):
    """"""

    Load = "Load"  #:
    Discharge = "Discharge"  #:
    All = "All"  #:


class CongestionVesselsOperation(Enum):
    """"""

    Load = "Load"  #:
    Discharge = "Discharge"  #:
    All = "All"  #:


class Platform(Enum):
    """"""

    LNG = "https://api-lng.kpler.com/v1"  #:
    LPG = "https://api-lpg.kpler.com/v1"  #:
    Dry = "https://api-coal.kpler.com/v1"  #:
    Liquids = "https://api.kpler.com/v1"  #:


class TradesStatus(Enum):
    """"""

    Delivered = "delivered"  #:
    Scheduled = "scheduled"  #:
    Loading = "loading"  #:
    InTransit = "in transit"  #:


class FleetDevelopmentSeriesAggregationMetric(Enum):
    """"""

    Count = "count"  #:
    SumCapacity = "sumcapacity"  #:
    SumDeadWeight = "sumdeadweight"  #:


class FleetDevelopmentSeriesMetric(Enum):
    """"""

    Available = "available"  #:
    Deliveries = "deliveries"  #:
    Scrapping = "scrapping"  #:
    Contracting = "contracting"  #:


class FleetDevelopmentSeriesPeriod(Enum):
    """"""

    Annually = "years"  #:
    Monthly = "months"  #:
    Quarterly = "quarters"  #:


class FleetDevelopmentSeriesSplit(Enum):
    """"""

    Total = "total"  #:
    ComplianceMethod = "complianceMethod"  #:
    VesselType = "vesselType"  #:
    VesselTypeOil = "vesselTypeOil"  #:
    VesselTypeCpp = "vesselTypeCpp"  #:


class FleetDevelopmentSeriesUnit(Enum):
    """"""

    MT = "mt"  #:
    KT = "kt"  #:


class FlowsDirection(Enum):
    """"""

    Import = "import"  #:
    Export = "export"  #:
    NetImport = "netimport"  #:
    NetExport = "netexport"  #:


class FlowsSplit(Enum):
    """"""

    Total = "total"  #:
    Grades = "grades"  #:
    Products = "products"  #:
    OriginCountries = "origin countries"  #:
    OriginSubcontinents = "origin subcontinents"  #:
    OriginContinents = "origin continents"  #:
    OriginTradingRegions = "origin trading regions"  #:
    OriginPorts = "origin ports"
    DestinationTradingRegions = "destination trading regions"  #:
    DestinationCountries = "destination countries"  #:
    DestinationSubcontinents = "destination subcontinents"  #:
    DestinationContinents = "destination continents"  #:
    OriginInstallations = "origin installations"  #:
    DestinationInstallations = "destination installations"  #:
    DestinationPorts = "destination ports"
    OriginPadds = "origin padds"  #:
    DestinationPadds = "destination padds"  #:
    VesselType = "vessel type"  #:
    TradeStatus = "trade status"  #:
    Sources = "sources"  #:
    Charterers = "charterers"  #:
    Routes = "routes"  #:
    Buyers = "buyer"  #:
    Sellers = "seller"  #:
    VesselTypeOil = "vessel type oil"  #:
    VesselTypeCpp = "vessel type cpp"  #:
    LongHaulVesselType = "long haul vessel type"  #:
    LongHaulVesselTypeOil = "long haul vessel type oil"  #:
    LongHaulVesselTypeCpp = "long haul vessel type cpp"  #:
    CrudeQuality = "crude quality"  #:


class FlowsPeriod(Enum):
    """"""

    Annually = "annually"  #:
    Monthly = "monthly"  #:
    Weekly = "weekly"  #:
    EiaWeekly = "eia-weekly"  #:
    Daily = "daily"  #:


class FlowsMeasurementUnit(Enum):
    """"""

    KBD = "kbd"  #:
    BBL = "bbl"  #:
    KB = "kb"  #:
    MMBBL = "mmbbl"  #:
    MT = "mt"  #:
    KT = "kt"  #:
    T = "t"  #:
    CM = "cm"  #:


class FreightMetricsSeriesMetric(Enum):
    """"""

    TonMiles = "TonMiles"  #:
    TonDays = "TonDays"  #:
    AvgSpeed = "AvgSpeed"  #:
    AvgDistance = "AvgDistance"  #:


class FreightMetricsSeriesPeriod(Enum):
    """"""

    Monthly = "monthly"  #:
    Quarterly = "quarterly"  #:
    Annually = "annually"  #:


class FreightMetricsSeriesSplit(Enum):
    """"""

    Total = "total"  #:
    DestinationCountry = "destinationCountry"  #:
    OriginCountry = "originCountry"  #:
    VesselType = "vesselType"  #:
    VesselTypeOil = "vesselTypeOil"  #:
    VesselTypeCpp = "vesselTypeCpp"  #:
    DestinationInstallation = "destinationInstallation"  #:
    OriginInstallation = "originInstallation"  #:
    DestinationSourceEta = "destinationSourceEta"  #:
    OriginSourceEta = "originSourceEta"  #:


class FreightMetricsSeriesUnit(Enum):
    """"""

    MTNMI = "mt/nmi"  #:
    KTNMI = "kt/nmi"  #:
    TNMI = "t/nmi"  #:
    TDAY = "t/day"  #:
    KTDAY = "kt/day"  #:


class FreightMetricsSeriesStatus(Enum):
    """Status of vessels for freight metrics series"""
    Ballast = "ballast"
    Loaded = "loaded"
    All = "all"


class VesselTypesDry(Enum):
    """"""

    BabyCapes = "Baby Capes"  #:
    Capesize = "Capesize"  #:
    Handymax = "Handymax"  #:
    Handysize = "Handysize"  #:
    Newcastlemax = "Newcastlemax"  #:
    Kamsarmax = "Kamsarmax"  #:
    Panamax = "Panamax"  #:
    PostPanamax = "Post-Panamax"  #:
    Supramax = "Supramax"  #:
    Ultramax = "Ultramax"  #:
    Valemax = "Valemax"  #:
    VLOC = "VLOC"  #:


class VesselTypesLPG(Enum):
    """"""

    SGC = "SGC"  #:
    VLGC = "VLGC"  #:
    Handysize = "Handysize"  #:
    MGC = "MGC"  #:
    LGC = "LGC"  #:
    VLEC = "VLEC"  #:


class VesselTypesLNG(Enum):
    """"""

    XLUpperConventional = "XL (Upper Conventional)"  #:
    LLowerConventional = "L (Lower Conventional)"  #:
    QFlex = "Q-Flex"  #:
    XSPressureGas = "XS (Pressure Gas)"  #:
    MMedMax = "M (Med Max)"  #:
    SSmallSCale = "S (Small Scale)"  #:
    QMax = "Q-Max"  #:


class VesselTypesCPP(Enum):
    """"""

    LR2 = "LR2"  #:
    VLCC = "VLCC"  #:
    LR3 = "LR3"  #:
    MR = "MR"  #:
    GP = "GP"  #:
    LR1 = "LR1"  #:
    SmallTanker = "Small Tanker"  #:


class VesselTypesOil(Enum):
    """"""

    Aframax = "Aframax"  #:
    ProductTanker = "Product Tanker"  #:
    Suezmax = "Suezmax"  #:
    VLCC = "VLCC"  #:
    Panamax = "Panamax"  #:
    ULCC = "ULCC"  #:
    SmallTanker = "Small Tanker"  #:


class FleetDevelopmentVesselsComplianceMethods(Enum):
    """"""

    Scrubber = "Scrubber"  #:
    ScrubberPlanned = "Scrubber Planned"  #:
    ScrubberReady = "Scrubber Ready"  #:


class FleetDevelopmentVesselsMetric(Enum):
    """"""

    Available = "available"  #:
    Deliveries = "deliveries"  #:
    Scrapping = "scrapping"  #:
    Contracting = "contracting"  #:


class FleetMetricsAlgo(Enum):
    """"""

    FloatingStorage = "floating_storage"  #:
    LoadedVessels = "loaded_vessels"  #:


class FleetMetricsPeriod(Enum):
    """"""

    Weekly = "weekly"  #:
    Daily = "daily"  #:
    EIA = "eia"  #:


class FleetMetricsMeasurementUnit(Enum):
    """"""

    BBL = "bbl"  #:
    T = "t"  #:
    CM = "cm"  #:


class FleetMetricsSplit(Enum):
    """"""

    Total = "total"  #:
    Grades = "grades"  #:
    Products = "products"  #:
    OriginCountries = "origin countries"  #:
    OriginTradingRegions = "origin trading regions"
    DestinationTradingRegions = "destination trading regions"
    OriginContinents = "origin continents"
    OriginSubcontinents = "origin subcontinents"
    DestinationSubcontinents = "destination subcontinents"
    OriginPorts = "origin ports"
    DestinationPorts = "destination ports"
    DestinationContinents = "destination continents"
    DestinationCountries = "destination countries"  #:
    OriginInstallations = "origin installations"  #:
    DestinationInstallations = "destination installations"  #:
    VesselType = "vessel type"  #:
    VesselTypeOil = "vessel type oil"
    VesselTypeCpp = "vessel type cpp"
    TradeStatus = "trade status"  #:
    Charterers = "charterers"  #:
    Buyer = "buyer"  #:
    Seller = "seller"  #:
    CurrentContinents = "current continents"  #:
    CurrentSubcontinents = "current subcontinents"  #:
    CurrentCountries = "current countries"  #:
    CurrentSubregions = "current subregions"  #:
    CurrentSeas = "current seas"  #:
    FloatingDays = "floating days"  #:
    CrudeQuality = "crude quality"  #:


class FleetMetricsVesselsAlgo(Enum):
    """"""

    FloatingStorage = "floating_storage"  #:
    LoadedVessels = "loaded_vessels"  #:


class FleetMetricsVesselsMeasurementUnit(Enum):
    """"""

    BBL = "bbl"  #:
    KB = "kb"  #:
    MMBBL = "mmbbl"  #:
    MT = "mt"  #:
    KT = "kt"  #:
    T = "t"  #:
    CM = "cm"  #:


class OutagesTypes(Enum):
    """"""

    Planned = "planned"  #:
    Unplanned = "unplanned"  #:


class ContractsTypes(Enum):
    """"""

    SPA = "SPA"  #:
    TUA = "TUA"  #:
    LTA = "LTA"  #:
    Tender = "Tender"  #:


class FleetUtilizationSeriesPeriod(Enum):
    """"""

    Annually = "years"  #:
    Monthly = "months"  #:
    Weekly = "weeks"  #:
    Quarterly = "quarters"  #:
    Daily = "days"  #:


class FleetUtilizationVesselsPeriod(Enum):
    """"""

    Annually = "years"  #:
    Monthly = "months"  #:
    Weekly = "weeks"  #:
    Quarterly = "quarters"  #:
    Daily = "days"  #:


class FleetUtilizationSeriesSplit(Enum):
    """"""

    Total = "total"  #:
    Product = "product"  #:
    State = "state"  #:
    VesselType = "vesselType"  #:
    VesselTypeOil = "vesselTypeOil"  #:
    VesselTypeCpp = "vesselTypeCpp"  #:
    SubState = "subState"  #:
    Direction = "direction"  #:
    CurrentSubContinents = "currentSubcontinents"  #:
    CurrentContinents = "currentContinents"  #:
    CurrentSubregions = "currentSubregions"  #:
    CurrentCountries = "currentCountries"  #:
    CurrentSeas = "currentSeas"  #:
    PreviousTradingRegions = "previousTradingRegions"  #:
    PreviousContinents = "previousContinents"  #:
    PreviousSubContinents = "previousSubContinents"  #:
    PreviousCountries = "previousCountries"  #:
    PreviousPorts = "previousPorts"  #:
    NextTradingRegions = "nextTradingRegions"  #:
    NextContinents = "nextContinents"  #:
    NextSubContinents = "nextSubContinents"  #:
    NextCountries = "nextCountries"  #:
    NextPorts = "nextPorts"  #:


class FleetUtilizationSeriesUnit(Enum):
    """"""

    MT = "mt"  #:
    KT = "kt"  #:


class FleetUtilizationSeriesVesselsState(Enum):
    """"""

    Ballast = "Ballast"  #:
    Loaded = "Loaded"  #:
    Maintenance = "Maintenance"  #:


class FleetUtilizationSeriesMetric(Enum):
    """"""

    Count = "count"  #:
    DeadWeight = "deadWeight"  #:


class FleetUtilizationVesselsState(Enum):
    """"""

    Ballast = "Ballast"  #:
    Loaded = "Loaded"  #:
    Maintenance = "Maintenance"  #:


class FleetUtilizationVesselsUnit(Enum):
    """"""

    MT = "mt"  #:
    KT = "kt"  #:


class InventoriesPeriod(Enum):
    """"""

    Annually = "annually"  #:
    Monthly = "monthly"  #:
    Weekly = "weekly"  #:
    EiaWeekly = "eia-weekly"  #:
    Daily = "daily"  #:


class InventoriesSplit(Enum):
    """"""

    Total = "total"  #:
    ByCountry = "byCountry"  #:
    ByInstallation = "byInstallation"  #:
    ByPlayer = "byPlayer"  #:
    ByTankType = "byTankType"  #:
    ByOnshoreOffshoreStatus = "byOnshoreOffshoreStatus"  #:


class InventoriesDronePeriod(Enum):
    """"""

    MidWeeks = "midweeks"  #:
    EiaWeeks = "eia"  #:
    Days = "days"  #:


class DiversionsVesselState(Enum):
    """"""

    Ballast = "Ballast"  #:
    Loaded = "Loaded"  #:


class SupplyDemandSplit(Enum):
    """"""

    Total = "total"  #:
    Country = "country"  #:


class SupplyDemandMetric(Enum):
    """"""

    Supply = "supply"  #:
    Demand = "demand"  #:
    RefineryRun = "refineryRun"  #:
    DirectCrudeUse = "directCrudeUse"  #:
    Balance = "balance"  #:
    NetExport = "netExport"  #:
    StockChange = "stockChange"  #:
    BalancingFactor = "balancingFactor"  #:


class SupplyDemandUsBalancesCrude(Enum):
    """"""

    Adjustment = "adjustment"  #:
    ImportFromCanada = "importFromCanada"  #:
    NetImport = "netImport"  #:
    RefineryRun = "refineryRun"  #:
    Production = "production"  #:
    StockChange = "stockChange"  #:


class SupplyDemandUsBalancesGasoline(Enum):
    """"""

    SupplyByBlender = "supplyByBlender"  #:
    Demand = "demand"  #:
    MaritimeImport = "maritimeImport"  #:
    NetReceipt = "netReceipt"  #:
    StockLevelPadd1A = "stockLevelPadd1A"  #:
    StockLevelPadd1B = "stockLevelPadd1B"  #:
    StockLevelPadd1C = "stockLevelPadd1C"  #:
    PipelineInbound = "pipelineInbound"  #:
    PipelineOutbound = "pipelineOutbound"  #:
    SupplyByRefinery = "supplyByRefinery"  #:
    WaterborneInbound = "waterborneInbound"  #:


class SupplyDemandUsBalancesGranularity(Enum):
    """"""

    Weekly = "weeks"  #:
    Monthly = "months"  #:


class SupplyDemandUsBalancesProduct(Enum):
    """"""

    Crude = "Crude/Co"  #:
    Gasoline = "Gasoline"  #:


class UtilizationGranularity(Enum):
    """"""

    Daily = "day"  #:
    Weekly = "week"  #:
    EiaWeekly = "eia-week"  #:
    Monthly = "month"  #:
    Annually = "year"  #:


class UtilizationSplit(Enum):
    """"""

    Total = "Total"  #:
    RefineryType = "Refinery Type"  #:
    Refinery = "Refinery"  #:
    Country = "Country"  #:
    Subcontinent = "Subcontinent"  #:
    Continent = "Continent"  #:
    TradingRegion = "Trading Region"  #:


class SecondaryFeedGranularity(Enum):
    """"""

    Daily = "day"  #:
    Weekly = "week"  #:
    EiaWeekly = "eia-week"  #:
    Monthly = "month"  #:
    Annually = "year"  #:


class SecondaryFeedSplit(Enum):
    """"""

    Total = "Total"  #:
    RefineryType = "Refinery Type"  #:
    Refinery = "Refinery"  #:
    Country = "Country"  #:
    Subcontinent = "Subcontinent"  #:
    Continent = "Continent"  #:
    TradingRegion = "Trading Region"  #:


class SecondaryFeedUnit(Enum):
    """"""

    KBD = "kbd"  #:
    KB = "kb"  #:
    M3 = "m3"  #:
    MBBL = "Mbbl"  #:


class MarginsGranularity(Enum):
    """"""

    Daily = "day"  #:
    Weekly = "week"  #:
    EiaWeekly = "eia-week"  #:
    Monthly = "month"  #:
    Annually = "year"  #:


class MarginsSplit(Enum):
    """"""

    Total = "Total"  #:
    RefineryType = "Refinery Type"  #:
    Refinery = "Refinery"  #:
    Country = "Country"  #:
    Subcontinent = "Subcontinent"  #:
    Continent = "Continent"  #:
    TradingRegion = "Trading Region"  #:


class FixturesStatuses(Enum):
    """"""

    OnSubs = "on subs"  #:
    InProgress = "in progress"  #:
    FullyFixed = "fully fixed"  #:
    Finished = "finished"  #:
    Failed = "failed"  #:
    Cancelled = "cancelled"  #:


class RefinedProductsGranularity(Enum):
    """"""

    Daily = "day"  #:
    Weekly = "week"  #:
    EiaWeekly = "eia-week"  #:
    Monthly = "month"  #:
    Annually = "year"  #:


class RefinedProducts(Enum):
    """"""

    All = "All"  #:
    TopOfTheBarrel = "Top of the Barrel"  #:
    LPG = "LPG"  #:
    Olefins = "Olefins"  #:
    Naphtha = "Naphtha"  #:
    Gasoline = "Gasoline"  #:
    MiddleOfTheBarrel = "Middle of the Barrel"  #:
    Jet = "Jet"  #:
    LowSulfurDiesel = "Low Sulfur Diesel"  #:
    HighSulfurDiesel = "High Sulfur Diesel"  #:
    BottomOfTheBarrel = "Bottom of the Barrel"  #:
    VGO = "VGO"  #:
    LowSulfurFuelOil = "Low Sulfur Fuel Oil"  #:
    HighSulfurFuelOil = "High Sulfur Fuel Oil"  #:
    Slurry = "Slurry"  #:
    AsphaltBitumen = "Asphalt/Bitumen"  #:
    LubeOils = "LubeOils"  #:
    Petcoke = "Petcoke"  #:


class RefinedProductsSplit(Enum):
    """"""

    Total = "Total"  #:
    RefineryType = "Refinery Type"  #:
    Refinery = "Refinery"  #:
    Country = "Country"  #:
    Subcontinent = "Subcontinent"  #:
    Continent = "Continent"  #:
    TradingRegion = "Trading Region"  #:
    Product = "Product"  #:


class RefinedProductsUnit(Enum):
    """"""

    KBD = "kbd"  #:
    KB = "kb"  #:
    M3 = "m3"  #:
    MBBL = "Mbbl"  #:
    KT = "kt"  #:
    PERCENTAGE = "percentage"  #:
    MT = "Mt"  #:


class RunsGranularity(Enum):
    """"""

    Daily = "day"  #:
    Weekly = "week"  #:
    EiaWeekly = "eia-week"  #:
    Monthly = "month"  #:
    Annually = "year"  #:


class RunsQualities(Enum):
    """"""

    All = "All"  #:
    Light = "Light"  #:
    LightSour = "Light Sour"  #:
    LightSweet = "Light Sweet"  #:
    Medium = "Medium"  #:
    MediumSour = "Medium Sour"  #:
    MediumSweet = "Medium Sweet"  #:
    Heavy = "Heavy"  #:
    HeavySour = "Heavy Sour"  #:
    HeavySweet = "Heavy Sweet"  #:
    Sweet = "Sweet"  #:
    Sour = "Sour"  #:


class RunsSplit(Enum):
    """"""

    Total = "Total"  #:
    RefineryType = "Refinery Type"  #:
    Refinery = "Refinery"  #:
    Country = "Country"  #:
    Subcontinent = "Subcontinent"  #:
    Continent = "Continent"  #:
    TradingRegion = "Trading Region"  #:
    CrudeQuality = "Crude Quality"  #:
    CrudeProductionRegion = "Crude Production Region"  #:


class ImportsSplit(Enum):
    """"""

    Total = "Total"  #:
    RefineryType = "Refinery Type"  #:
    Refinery = "Refinery"  #:
    Country = "Country"  #:
    Subcontinent = "Subcontinent"  #:
    Continent = "Continent"  #:
    TradingRegion = "Trading Region"  #:
    CrudeQuality = "Crude Quality"  #:

    CrudeProductionRegion = "Crude Production Region"  #:
class ImportsMetric(Enum):
    """"""

    Quantity = "Quantity"  #:
    APISulfur = "API Sulfur"  #:

class ImportsGranularity(Enum):
    """"""

    Daily = "day"  #:
    Weekly = "week"  #:
    EiaWeekly = "eia-week"  #:
    Monthly = "month"  #:
    Annually = "year"  #:


class RunsUnit(Enum):
    """"""

    KBD = "kbd"  #:
    KB = "kb"  #:
    M3 = "m3"  #:
    MBBL = "Mbbl"  #:


class ImportsUnit(Enum):
    """"""

    KBD = "kbd"  #:
    KB = "kb"  #:
    M3 = "m3"  #:
    MBBL = "Mbbl"  #:



from datetime import date
from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import (
    process_date_parameter,
    process_enum_parameters,
    process_list_parameter,
)


class Outages(KplerClient):

    """
    The ``Outages`` endpoint allows users to view all current, future, and historic outages, planned and unplanned,
    going back to 2016.
    """

    RESOURCE_NAME = "outages"

    AVAILABLE_PLATFORMS = [Platform.LNG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint outages.

        Examples:
            >>> from kpler.sdk.resources.outages import Outages
            ... outages_client = Outages(config)
            ... outages_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "outagetype","Outages Type","Planned or unplanned","False","string"
                "installationName","Installation Name","None","False","string"
                "installationCountry","Installation Country","None","False","string"
                "start","Start","None","False","datetime yyyy-MM-dd HH:mm"
                "end","End","None","False","datetime yyyy-MM-dd HH:mm"
                "comment","Comment","None","False","string"
                "installationId","Installation Id","Identifier in the database of Kpler","False","long"
        """

        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self,
        size: Optional[int] = None,
        start_date: Optional[date] = None,
        end_date: Optional[date] = None,
        installations: Optional[List[str]] = None,
        zones: Optional[List[str]] = None,
        outage_types: Optional[List[str]] = None,
        columns: Optional[List[str]] = None,
    ):
        """

        Args:
            size: Optional[int] Maximum number of outages returned
            start_date: Optional[date] Start of the period (YYYY-MM-DD)
            end_date: Optional[date] End of the period (YYYY-MM-DD)
            installations: Optional[List[str]] Names of installations
            zones: Optional[List[str]] Names of zones ["port", "region", "country", "continent"]
            outage_types: Optional[List[Enum]] = ``OutagesTypes``
            columns: Optional[List[str]] Retrieve all available columns when set to "all"

        Examples:
            >>> from kpler.sdk.resources.outages import Outages
            ... from kpler.sdk import OutagesTypes
            ... outages_client = Outages(config)
            ... outages_client.get(
            ...     installations=["Sabine Pass"],
            ...     zones=["United States"],
            ...     size=10,
            ...     outage_types=[OutagesTypes.Planned]
            ... )

            .. csv-table::
                :header: "outagetype","installationName","installationCountry","start","end","comment","installationId"

                "Planned","Sabine Pass","United States","2020-02-10 00:00","2020-02-14 00:00","Compressor and electrical maintenance of the G...","3605"
                "Planned","Sabine Pass","United States","2019-09-01 16:05","2019-09-20 16:05","Scheduled turnarounds on train 5","3605"
                "Planned","Sabine Pass","United States","2019-08-05 00:00","2019-08-13 00:00","Maintenance at the Gillis compressor station r...","3605"
                "Planned","Sabine Pass","United States","2019-08-05 00:00","2019-08-26 00:00","Scheduled turnarounds on train 3 and 4","3605"
                "Planned","Sabine Pass","United States","2019-04-01 00:00","2019-04-11 00:00","2 trains are down for maintenance (T3 & T4)","3605"
                "Planned","Sabine Pass","United States","2019-03-22 00:00","2019-04-01 00:00","2 trains are under maintenance (T1&T2)","3605"
                "Planned","Sabine Pass","United States","2017-07-31 11:12","2017-09-15 09:32","The outages is on Train 3, with the end date to...","3605"
                "Planned","Sabine Pass","United States","2016-09-17 00:00","2016-10-26 00:00","Train 1 & 2 Shutdown","3605"
        """

        query_parameters = {
            "size": size,
            "startDate": process_date_parameter(start_date),
            "endDate": process_date_parameter(end_date),
            "installations": process_list_parameter(installations),
            "zones": process_list_parameter(zones),
            "outageTypes": process_enum_parameters(outage_types),
            "columns": process_list_parameter(columns),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)



from typing import List, Optional

from pandas import DataFrame

from kpler.sdk import Platform
from kpler.sdk.client import KplerClient
from kpler.sdk.configuration import Configuration
from kpler.sdk.helpers import process_bool_parameter, process_list_parameter


class Vessels(KplerClient):
    """
    The ``Vessels`` query returns a snapshot of the current status of the fleet, including details on vessels status,
    last port call and next destination.
    """

    RESOURCE_NAME = "vessels"

    AVAILABLE_PLATFORMS = [Platform.Dry, Platform.Liquids, Platform.LNG, Platform.LPG]

    def __init__(self, configuration: Configuration, column_ids: bool = True, log_level=None):
        super().__init__(configuration, self.AVAILABLE_PLATFORMS, column_ids, log_level)

    def get_columns(self) -> DataFrame:
        """
        This endpoint returns a recent and updated list of all columns available for the endpoint vessels.

        Examples:
            >>> from kpler.sdk.resources.vessels import Vessels
            ... vessels_client = Vessels(config)
            ... vessels_client.get_columns()

            .. csv-table::
                :header: "id","name","description","deprecated","type"

                "vessel_name","Vessels","Name of the vessels","False","string"
                "vessel_status","Status","Status of the vessels","False","string"
                "last_port_call_location_name","Last port-call","Vessels's last port call location","False","string"
                "last_port_call_end","Departure date","End of the last port call","False","datetime yyyy-MM-dd HH:mm"
                "cargo_on_board_cubic_meters","Volume","None","False","double"
                "...","...","...","...","..."
        """
        return self._get_columns_for_resource(self.RESOURCE_NAME)

    def get(
        self, columns: Optional[List[str]] = None, with_freight_view: bool = False
    ) -> DataFrame:
        """

        Args:
             columns: Optional[List[str]] Retrieve all available columns when set to "all"
             with_freight_view: bool By default: with_freight_view=False. Provides access to the entire fleet's trades, irrespective of your current cargo subscription. Only available via Freight subscription.

        Examples:
            >>> from kpler.sdk.resources.vessels import Vessels
            ... vessels_client = Vessels(config)
            ... vessels_client.get(columns=["vessel_name","vessel_status","last_port_call_end"])

            .. csv-table::
                :header: "vessel_name","vessel_status","last_port_call_location_name","next_destination_zone_name"

                "...","...","...","..."
                "Anichkov Bridge","Active","Pelepas Light.","Western Petroleum B Anchorage"
                "Anika","Active","PT SON Edible Oil Refinery","Lahad Datu"
                "Anikitos","Inactive","",""
                "Anikitos","Inactive","",""
                "Anikitos","Active","Rio Grande","Barcarena"
                "...","...","...","..."
        """
        query_parameters = {
            "columns": process_list_parameter(columns),
            "withFreightView": process_bool_parameter(with_freight_view),
        }
        return self._get_dataframe(self.RESOURCE_NAME, query_parameters)

    def search(self, q: str) -> DataFrame:
        """
        Performs the vessels name search by the given arguments

        Args:
            q: str Argument to search by in vessels names

        Examples:
            >>> from kpler.sdk.resources.vessels import Vessels
            ... vessels_client = Vessels(config)
            ... vessels_client.search("Ab")

            .. csv-table::
                :header: "vessels"

                "Abu Dhabi-Iii"
                "Bolan"
                "D&K Abdul Razzak Khalid Zaid Al-Khalid"
                "Abalone"
                "Abdias Nascimento"
                "..."
        """
        query_parameters = {"q": q, "resources": self.RESOURCE_NAME}
        return self._search(query_parameters)
